@@ -15,12 +15,17 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "dead_branch_elim_pass.h"
+#include "source/opt/dead_branch_elim_pass.h"
 
-#include "cfa.h"
-#include "ir_context.h"
-#include "iterator.h"
-#include "make_unique.h"
+#include <list>
+#include <memory>
+#include <vector>
+
+#include "source/cfa.h"
+#include "source/opt/ir_context.h"
+#include "source/opt/iterator.h"
+#include "source/opt/struct_cfg_analysis.h"
+#include "source/util/make_unique.h"
 
 namespace spvtools {
 namespace opt {
@@ -34,7 +39,7 @@ const uint32_t kBranchCondFalseLabIdInIdx = 2;
 
 bool DeadBranchElimPass::GetConstCondition(uint32_t condId, bool* condVal) {
   bool condIsConst;
-  ir::Instruction* cInst = get_def_use_mgr()->GetDef(condId);
+  Instruction* cInst = get_def_use_mgr()->GetDef(condId);
   switch (cInst->opcode()) {
     case SpvOpConstantFalse: {
       *condVal = false;
@@ -56,9 +61,9 @@ bool DeadBranchElimPass::GetConstCondition(uint32_t condId, bool* condVal) {
 }
 
 bool DeadBranchElimPass::GetConstInteger(uint32_t selId, uint32_t* selVal) {
-  ir::Instruction* sInst = get_def_use_mgr()->GetDef(selId);
+  Instruction* sInst = get_def_use_mgr()->GetDef(selId);
   uint32_t typeId = sInst->type_id();
-  ir::Instruction* typeInst = get_def_use_mgr()->GetDef(typeId);
+  Instruction* typeInst = get_def_use_mgr()->GetDef(typeId);
   if (!typeInst || (typeInst->opcode() != SpvOpTypeInt)) return false;
   // TODO(greg-lunarg): Support non-32 bit ints
   if (typeInst->GetSingleWordInOperand(0) != 32) return false;
@@ -72,27 +77,30 @@ bool DeadBranchElimPass::GetConstInteger(uint32_t selId, uint32_t* selVal) {
   return false;
 }
 
-void DeadBranchElimPass::AddBranch(uint32_t labelId, ir::BasicBlock* bp) {
+void DeadBranchElimPass::AddBranch(uint32_t labelId, BasicBlock* bp) {
   assert(get_def_use_mgr()->GetDef(labelId) != nullptr);
-  std::unique_ptr<ir::Instruction> newBranch(new ir::Instruction(
-      context(), SpvOpBranch, 0, 0,
-      {{spv_operand_type_t::SPV_OPERAND_TYPE_ID, {labelId}}}));
-  get_def_use_mgr()->AnalyzeInstDefUse(&*newBranch);
+  std::unique_ptr<Instruction> newBranch(
+      new Instruction(context(), SpvOpBranch, 0, 0,
+                      {{spv_operand_type_t::SPV_OPERAND_TYPE_ID, {labelId}}}));
+  context()->AnalyzeDefUse(&*newBranch);
+  context()->set_instr_block(&*newBranch, bp);
   bp->AddInstruction(std::move(newBranch));
 }
 
-ir::BasicBlock* DeadBranchElimPass::GetParentBlock(uint32_t id) {
+BasicBlock* DeadBranchElimPass::GetParentBlock(uint32_t id) {
   return context()->get_instr_block(get_def_use_mgr()->GetDef(id));
 }
 
 bool DeadBranchElimPass::MarkLiveBlocks(
-    ir::Function* func, std::unordered_set<ir::BasicBlock*>* live_blocks) {
-  std::unordered_set<ir::BasicBlock*> continues;
-  std::vector<ir::BasicBlock*> stack;
+    Function* func, std::unordered_set<BasicBlock*>* live_blocks) {
+  StructuredCFGAnalysis* cfgAnalysis = context()->GetStructuredCFGAnalysis();
+
+  std::unordered_set<BasicBlock*> continues;
+  std::vector<BasicBlock*> stack;
   stack.push_back(&*func->begin());
   bool modified = false;
   while (!stack.empty()) {
-    ir::BasicBlock* block = stack.back();
+    BasicBlock* block = stack.back();
     stack.pop_back();
 
     // Live blocks doubles as visited set.
@@ -101,7 +109,7 @@ bool DeadBranchElimPass::MarkLiveBlocks(
     uint32_t cont_id = block->ContinueBlockIdIfAny();
     if (cont_id != 0) continues.insert(GetParentBlock(cont_id));
 
-    ir::Instruction* terminator = block->terminator();
+    Instruction* terminator = block->terminator();
     uint32_t live_lab_id = 0;
     // Check if the terminator has a single valid successor.
     if (terminator->opcode() == SpvOpBranchConditional) {
@@ -152,9 +160,20 @@ bool DeadBranchElimPass::MarkLiveBlocks(
       // Remove the merge instruction if it is a selection merge.
       AddBranch(live_lab_id, block);
       context()->KillInst(terminator);
-      ir::Instruction* mergeInst = block->GetMergeInst();
+      Instruction* mergeInst = block->GetMergeInst();
       if (mergeInst && mergeInst->opcode() == SpvOpSelectionMerge) {
-        context()->KillInst(mergeInst);
+        Instruction* first_break = FindFirstExitFromSelectionMerge(
+            live_lab_id, mergeInst->GetSingleWordInOperand(0),
+            cfgAnalysis->LoopMergeBlock(live_lab_id),
+            cfgAnalysis->LoopContinueBlock(live_lab_id));
+        if (first_break == nullptr) {
+          context()->KillInst(mergeInst);
+        } else {
+          mergeInst->RemoveFromList();
+          first_break->InsertBefore(std::unique_ptr<Instruction>(mergeInst));
+          context()->set_instr_block(mergeInst,
+                                     context()->get_instr_block(first_break));
+        }
       }
       stack.push_back(GetParentBlock(live_lab_id));
     } else {
@@ -170,18 +189,17 @@ bool DeadBranchElimPass::MarkLiveBlocks(
 }
 
 void DeadBranchElimPass::MarkUnreachableStructuredTargets(
-    const std::unordered_set<ir::BasicBlock*>& live_blocks,
-    std::unordered_set<ir::BasicBlock*>* unreachable_merges,
-    std::unordered_map<ir::BasicBlock*, ir::BasicBlock*>*
-        unreachable_continues) {
+    const std::unordered_set<BasicBlock*>& live_blocks,
+    std::unordered_set<BasicBlock*>* unreachable_merges,
+    std::unordered_map<BasicBlock*, BasicBlock*>* unreachable_continues) {
   for (auto block : live_blocks) {
     if (auto merge_id = block->MergeBlockIdIfAny()) {
-      ir::BasicBlock* merge_block = GetParentBlock(merge_id);
+      BasicBlock* merge_block = GetParentBlock(merge_id);
       if (!live_blocks.count(merge_block)) {
         unreachable_merges->insert(merge_block);
       }
       if (auto cont_id = block->ContinueBlockIdIfAny()) {
-        ir::BasicBlock* cont_block = GetParentBlock(cont_id);
+        BasicBlock* cont_block = GetParentBlock(cont_id);
         if (!live_blocks.count(cont_block)) {
           (*unreachable_continues)[cont_block] = block;
         }
@@ -191,9 +209,8 @@ void DeadBranchElimPass::MarkUnreachableStructuredTargets(
 }
 
 bool DeadBranchElimPass::FixPhiNodesInLiveBlocks(
-    ir::Function* func, const std::unordered_set<ir::BasicBlock*>& live_blocks,
-    const std::unordered_map<ir::BasicBlock*, ir::BasicBlock*>&
-        unreachable_continues) {
+    Function* func, const std::unordered_set<BasicBlock*>& live_blocks,
+    const std::unordered_map<BasicBlock*, BasicBlock*>& unreachable_continues) {
   bool modified = false;
   for (auto& block : *func) {
     if (live_blocks.count(&block)) {
@@ -204,8 +221,8 @@ bool DeadBranchElimPass::FixPhiNodesInLiveBlocks(
 
         bool changed = false;
         bool backedge_added = false;
-        ir::Instruction* inst = &*iter;
-        std::vector<ir::Operand> operands;
+        Instruction* inst = &*iter;
+        std::vector<Operand> operands;
         // Build a complete set of operands (not just input operands). Start
         // with type and result id operands.
         operands.push_back(inst->GetOperand(0u));
@@ -218,7 +235,7 @@ bool DeadBranchElimPass::FixPhiNodesInLiveBlocks(
         // However, if there is only one other incoming edge, the OpPhi can be
         // eliminated.
         for (uint32_t i = 1; i < inst->NumInOperands(); i += 2) {
-          ir::BasicBlock* inc = GetParentBlock(inst->GetSingleWordInOperand(i));
+          BasicBlock* inc = GetParentBlock(inst->GetSingleWordInOperand(i));
           auto cont_iter = unreachable_continues.find(inc);
           if (cont_iter != unreachable_continues.end() &&
               cont_iter->second == &block && inst->NumInOperands() > 4) {
@@ -301,10 +318,9 @@ bool DeadBranchElimPass::FixPhiNodesInLiveBlocks(
 }
 
 bool DeadBranchElimPass::EraseDeadBlocks(
-    ir::Function* func, const std::unordered_set<ir::BasicBlock*>& live_blocks,
-    const std::unordered_set<ir::BasicBlock*>& unreachable_merges,
-    const std::unordered_map<ir::BasicBlock*, ir::BasicBlock*>&
-        unreachable_continues) {
+    Function* func, const std::unordered_set<BasicBlock*>& live_blocks,
+    const std::unordered_set<BasicBlock*>& unreachable_merges,
+    const std::unordered_map<BasicBlock*, BasicBlock*>& unreachable_continues) {
   bool modified = false;
   for (auto ebi = func->begin(); ebi != func->end();) {
     if (unreachable_merges.count(&*ebi)) {
@@ -314,8 +330,10 @@ bool DeadBranchElimPass::EraseDeadBlocks(
         KillAllInsts(&*ebi, false);
         // Add unreachable terminator.
         ebi->AddInstruction(
-            MakeUnique<ir::Instruction>(context(), SpvOpUnreachable, 0, 0,
-                                        std::initializer_list<ir::Operand>{}));
+            MakeUnique<Instruction>(context(), SpvOpUnreachable, 0, 0,
+                                    std::initializer_list<Operand>{}));
+        context()->AnalyzeUses(ebi->terminator());
+        context()->set_instr_block(ebi->terminator(), &*ebi);
         modified = true;
       }
       ++ebi;
@@ -328,11 +346,11 @@ bool DeadBranchElimPass::EraseDeadBlocks(
         KillAllInsts(&*ebi, false);
         // Add unconditional branch to header.
         assert(unreachable_continues.count(&*ebi));
-        ebi->AddInstruction(
-            MakeUnique<ir::Instruction>(context(), SpvOpBranch, 0, 0,
-                                        std::initializer_list<ir::Operand>{
-                                            {SPV_OPERAND_TYPE_ID, {cont_id}}}));
+        ebi->AddInstruction(MakeUnique<Instruction>(
+            context(), SpvOpBranch, 0, 0,
+            std::initializer_list<Operand>{{SPV_OPERAND_TYPE_ID, {cont_id}}}));
         get_def_use_mgr()->AnalyzeInstUse(&*ebi->tail());
+        context()->set_instr_block(&*ebi->tail(), &*ebi);
         modified = true;
       }
       ++ebi;
@@ -349,13 +367,13 @@ bool DeadBranchElimPass::EraseDeadBlocks(
   return modified;
 }
 
-bool DeadBranchElimPass::EliminateDeadBranches(ir::Function* func) {
+bool DeadBranchElimPass::EliminateDeadBranches(Function* func) {
   bool modified = false;
-  std::unordered_set<ir::BasicBlock*> live_blocks;
+  std::unordered_set<BasicBlock*> live_blocks;
   modified |= MarkLiveBlocks(func, &live_blocks);
 
-  std::unordered_set<ir::BasicBlock*> unreachable_merges;
-  std::unordered_map<ir::BasicBlock*, ir::BasicBlock*> unreachable_continues;
+  std::unordered_set<BasicBlock*> unreachable_merges;
+  std::unordered_map<BasicBlock*, BasicBlock*> unreachable_continues;
   MarkUnreachableStructuredTargets(live_blocks, &unreachable_merges,
                                    &unreachable_continues);
   modified |= FixPhiNodesInLiveBlocks(func, live_blocks, unreachable_continues);
@@ -365,73 +383,159 @@ bool DeadBranchElimPass::EliminateDeadBranches(ir::Function* func) {
   return modified;
 }
 
-void DeadBranchElimPass::Initialize(ir::IRContext* c) {
-  InitializeProcessing(c);
+void DeadBranchElimPass::FixBlockOrder() {
+  context()->BuildInvalidAnalyses(IRContext::kAnalysisCFG |
+                                  IRContext::kAnalysisDominatorAnalysis);
+  // Reorders blocks according to DFS of dominator tree.
+  ProcessFunction reorder_dominators = [this](Function* function) {
+    DominatorAnalysis* dominators = context()->GetDominatorAnalysis(function);
+    std::vector<BasicBlock*> blocks;
+    for (auto iter = dominators->GetDomTree().begin();
+         iter != dominators->GetDomTree().end(); ++iter) {
+      if (iter->id() != 0) {
+        blocks.push_back(iter->bb_);
+      }
+    }
+    for (uint32_t i = 1; i < blocks.size(); ++i) {
+      function->MoveBasicBlockToAfter(blocks[i]->id(), blocks[i - 1]);
+    }
+    return true;
+  };
 
-  // Initialize extension whitelist
-  InitExtensions();
-};
+  // Reorders blocks according to structured order.
+  ProcessFunction reorder_structured = [this](Function* function) {
+    std::list<BasicBlock*> order;
+    context()->cfg()->ComputeStructuredOrder(function, &*function->begin(),
+                                             &order);
+    std::vector<BasicBlock*> blocks;
+    for (auto block : order) {
+      blocks.push_back(block);
+    }
+    for (uint32_t i = 1; i < blocks.size(); ++i) {
+      function->MoveBasicBlockToAfter(blocks[i]->id(), blocks[i - 1]);
+    }
+    return true;
+  };
 
-bool DeadBranchElimPass::AllExtensionsSupported() const {
-  // If any extension not in whitelist, return false
-  for (auto& ei : get_module()->extensions()) {
-    const char* extName =
-        reinterpret_cast<const char*>(&ei.GetInOperand(0).words[0]);
-    if (extensions_whitelist_.find(extName) == extensions_whitelist_.end())
-      return false;
+  // Structured order is more intuitive so use it where possible.
+  if (context()->get_feature_mgr()->HasCapability(SpvCapabilityShader)) {
+    context()->ProcessReachableCallTree(reorder_structured);
+  } else {
+    context()->ProcessReachableCallTree(reorder_dominators);
   }
-  return true;
 }
 
-Pass::Status DeadBranchElimPass::ProcessImpl() {
+Pass::Status DeadBranchElimPass::Process() {
   // Do not process if module contains OpGroupDecorate. Additional
   // support required in KillNamesAndDecorates().
   // TODO(greg-lunarg): Add support for OpGroupDecorate
   for (auto& ai : get_module()->annotations())
     if (ai.opcode() == SpvOpGroupDecorate) return Status::SuccessWithoutChange;
-  // Do not process if any disallowed extensions are enabled
-  if (!AllExtensionsSupported()) return Status::SuccessWithoutChange;
   // Process all entry point functions
-  ProcessFunction pfn = [this](ir::Function* fp) {
+  ProcessFunction pfn = [this](Function* fp) {
     return EliminateDeadBranches(fp);
   };
-  bool modified = ProcessReachableCallTree(pfn, context());
+  bool modified = context()->ProcessReachableCallTree(pfn);
+  if (modified) FixBlockOrder();
   return modified ? Status::SuccessWithChange : Status::SuccessWithoutChange;
 }
 
-DeadBranchElimPass::DeadBranchElimPass() {}
+Instruction* DeadBranchElimPass::FindFirstExitFromSelectionMerge(
+    uint32_t start_block_id, uint32_t merge_block_id, uint32_t loop_merge_id,
+    uint32_t loop_continue_id) {
+  // To find the "first" exit, we follow branches looking for a conditional
+  // branch that is not in a nested construct and is not the header of a new
+  // construct.  We follow the control flow from |start_block_id| to find the
+  // first one.
+  while (start_block_id != merge_block_id && start_block_id != loop_merge_id &&
+         start_block_id != loop_continue_id) {
+    BasicBlock* start_block = context()->get_instr_block(start_block_id);
+    Instruction* branch = start_block->terminator();
+    uint32_t next_block_id = 0;
+    switch (branch->opcode()) {
+      case SpvOpBranchConditional:
+        next_block_id = start_block->MergeBlockIdIfAny();
+        if (next_block_id == 0) {
+          // If a possible target is the |loop_merge_id| or |loop_continue_id|,
+          // which are not the current merge node, then we continue the search
+          // with the other target.
+          for (uint32_t i = 1; i < 3; i++) {
+            if (branch->GetSingleWordInOperand(i) == loop_merge_id &&
+                loop_merge_id != merge_block_id) {
+              next_block_id = branch->GetSingleWordInOperand(3 - i);
+              break;
+            }
+            if (branch->GetSingleWordInOperand(i) == loop_continue_id &&
+                loop_continue_id != merge_block_id) {
+              next_block_id = branch->GetSingleWordInOperand(3 - i);
+              break;
+            }
+          }
 
-Pass::Status DeadBranchElimPass::Process(ir::IRContext* module) {
-  Initialize(module);
-  return ProcessImpl();
-}
+          if (next_block_id == 0) {
+            return branch;
+          }
+        }
+        break;
+      case SpvOpSwitch:
+        next_block_id = start_block->MergeBlockIdIfAny();
+        if (next_block_id == 0) {
+          // A switch with no merge instructions can have at most 4 targets:
+          //   a. |merge_block_id|
+          //   b. |loop_merge_id|
+          //   c. |loop_continue_id|
+          //   d. 1 block inside the current region.
+          //
+          // This leads to a number of cases of what to do.
+          //
+          // 1. Does not jump to a block inside of the current construct.  In
+          // this case, there is not conditional break, so we should return
+          // |nullptr|.
+          //
+          // 2. Jumps to |merge_block_id| and a block inside the current
+          // construct.  In this case, this branch conditionally break to the
+          // end of the current construct, so return the current branch.
+          //
+          // 3.  Otherwise, this branch may break, but not to the current merge
+          // block.  So we continue with the block that is inside the loop.
 
-void DeadBranchElimPass::InitExtensions() {
-  extensions_whitelist_.clear();
-  extensions_whitelist_.insert({
-      "SPV_AMD_shader_explicit_vertex_parameter",
-      "SPV_AMD_shader_trinary_minmax",
-      "SPV_AMD_gcn_shader",
-      "SPV_KHR_shader_ballot",
-      "SPV_AMD_shader_ballot",
-      "SPV_AMD_gpu_shader_half_float",
-      "SPV_KHR_shader_draw_parameters",
-      "SPV_KHR_subgroup_vote",
-      "SPV_KHR_16bit_storage",
-      "SPV_KHR_device_group",
-      "SPV_KHR_multiview",
-      "SPV_NVX_multiview_per_view_attributes",
-      "SPV_NV_viewport_array2",
-      "SPV_NV_stereo_view_rendering",
-      "SPV_NV_sample_mask_override_coverage",
-      "SPV_NV_geometry_shader_passthrough",
-      "SPV_AMD_texture_gather_bias_lod",
-      "SPV_KHR_storage_buffer_storage_class",
-      "SPV_KHR_variable_pointers",
-      "SPV_AMD_gpu_shader_int16",
-      "SPV_KHR_post_depth_coverage",
-      "SPV_KHR_shader_atomic_counter_ops",
-  });
+          bool found_break = false;
+          for (uint32_t i = 1; i < branch->NumInOperands(); i += 2) {
+            uint32_t target = branch->GetSingleWordInOperand(i);
+            if (target == merge_block_id) {
+              found_break = true;
+            } else if (target != loop_merge_id && target != loop_continue_id) {
+              next_block_id = branch->GetSingleWordInOperand(i);
+            }
+          }
+
+          if (next_block_id == 0) {
+            // Case 1.
+            return nullptr;
+          }
+
+          if (found_break) {
+            // Case 2.
+            return branch;
+          }
+
+          // The fall through is case 3.
+        }
+        break;
+      case SpvOpBranch:
+        // Need to check if this is the header of a loop nested in the
+        // selection construct.
+        next_block_id = start_block->MergeBlockIdIfAny();
+        if (next_block_id == 0) {
+          next_block_id = branch->GetSingleWordInOperand(0);
+        }
+        break;
+      default:
+        return nullptr;
+    }
+    start_block_id = next_block_id;
+  }
+  return nullptr;
 }
 
 }  // namespace opt
