@@ -12,17 +12,17 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "instruction.h"
+#include "source/opt/instruction.h"
 
 #include <initializer_list>
 
-#include "disassemble.h"
-#include "fold.h"
-#include "ir_context.h"
-#include "reflect.h"
+#include "source/disassemble.h"
+#include "source/opt/fold.h"
+#include "source/opt/ir_context.h"
+#include "source/opt/reflect.h"
 
 namespace spvtools {
-namespace ir {
+namespace opt {
 
 namespace {
 // Indices used to get particular operands out of instructions using InOperand.
@@ -36,24 +36,24 @@ Instruction::Instruction(IRContext* c)
     : utils::IntrusiveNodeBase<Instruction>(),
       context_(c),
       opcode_(SpvOpNop),
-      type_id_(0),
-      result_id_(0),
+      has_type_id_(false),
+      has_result_id_(false),
       unique_id_(c->TakeNextUniqueId()) {}
 
 Instruction::Instruction(IRContext* c, SpvOp op)
     : utils::IntrusiveNodeBase<Instruction>(),
       context_(c),
       opcode_(op),
-      type_id_(0),
-      result_id_(0),
+      has_type_id_(false),
+      has_result_id_(false),
       unique_id_(c->TakeNextUniqueId()) {}
 
 Instruction::Instruction(IRContext* c, const spv_parsed_instruction_t& inst,
                          std::vector<Instruction>&& dbg_line)
     : context_(c),
       opcode_(static_cast<SpvOp>(inst.opcode)),
-      type_id_(inst.type_id),
-      result_id_(inst.result_id),
+      has_type_id_(inst.type_id != 0),
+      has_result_id_(inst.result_id != 0),
       unique_id_(c->TakeNextUniqueId()),
       dbg_line_insts_(std::move(dbg_line)) {
   assert((!IsDebugLineInst(opcode_) || dbg_line.empty()) &&
@@ -68,22 +68,21 @@ Instruction::Instruction(IRContext* c, const spv_parsed_instruction_t& inst,
 }
 
 Instruction::Instruction(IRContext* c, SpvOp op, uint32_t ty_id,
-                         uint32_t res_id,
-                         const std::vector<Operand>& in_operands)
+                         uint32_t res_id, const OperandList& in_operands)
     : utils::IntrusiveNodeBase<Instruction>(),
       context_(c),
       opcode_(op),
-      type_id_(ty_id),
-      result_id_(res_id),
+      has_type_id_(ty_id != 0),
+      has_result_id_(res_id != 0),
       unique_id_(c->TakeNextUniqueId()),
       operands_() {
-  if (type_id_ != 0) {
+  if (has_type_id_) {
     operands_.emplace_back(spv_operand_type_t::SPV_OPERAND_TYPE_TYPE_ID,
-                           std::initializer_list<uint32_t>{type_id_});
+                           std::initializer_list<uint32_t>{ty_id});
   }
-  if (result_id_ != 0) {
+  if (has_result_id_) {
     operands_.emplace_back(spv_operand_type_t::SPV_OPERAND_TYPE_RESULT_ID,
-                           std::initializer_list<uint32_t>{result_id_});
+                           std::initializer_list<uint32_t>{res_id});
   }
   operands_.insert(operands_.end(), in_operands.begin(), in_operands.end());
 }
@@ -91,16 +90,16 @@ Instruction::Instruction(IRContext* c, SpvOp op, uint32_t ty_id,
 Instruction::Instruction(Instruction&& that)
     : utils::IntrusiveNodeBase<Instruction>(),
       opcode_(that.opcode_),
-      type_id_(that.type_id_),
-      result_id_(that.result_id_),
+      has_type_id_(that.has_type_id_),
+      has_result_id_(that.has_result_id_),
       unique_id_(that.unique_id_),
       operands_(std::move(that.operands_)),
       dbg_line_insts_(std::move(that.dbg_line_insts_)) {}
 
 Instruction& Instruction::operator=(Instruction&& that) {
   opcode_ = that.opcode_;
-  type_id_ = that.type_id_;
-  result_id_ = that.result_id_;
+  has_type_id_ = that.has_type_id_;
+  has_result_id_ = that.has_result_id_;
   unique_id_ = that.unique_id_;
   operands_ = std::move(that.operands_);
   dbg_line_insts_ = std::move(that.dbg_line_insts_);
@@ -110,8 +109,8 @@ Instruction& Instruction::operator=(Instruction&& that) {
 Instruction* Instruction::Clone(IRContext* c) const {
   Instruction* clone = new Instruction(c);
   clone->opcode_ = opcode_;
-  clone->type_id_ = type_id_;
-  clone->result_id_ = result_id_;
+  clone->has_type_id_ = has_type_id_;
+  clone->has_result_id_ = has_result_id_;
   clone->unique_id_ = c->TakeNextUniqueId();
   clone->operands_ = operands_;
   clone->dbg_line_insts_ = dbg_line_insts_;
@@ -139,15 +138,14 @@ void Instruction::ToBinaryWithoutAttachedDebugInsts(
     binary->insert(binary->end(), operand.words.begin(), operand.words.end());
 }
 
-void Instruction::ReplaceOperands(const std::vector<Operand>& new_operands) {
+void Instruction::ReplaceOperands(const OperandList& new_operands) {
   operands_.clear();
   operands_.insert(operands_.begin(), new_operands.begin(), new_operands.end());
-  operands_.shrink_to_fit();
 }
 
 bool Instruction::IsReadOnlyLoad() const {
   if (IsLoad()) {
-    ir::Instruction* address_def = GetBaseAddress();
+    Instruction* address_def = GetBaseAddress();
     if (!address_def || address_def->opcode() != SpvOpVariable) {
       return false;
     }
@@ -158,12 +156,13 @@ bool Instruction::IsReadOnlyLoad() const {
 
 Instruction* Instruction::GetBaseAddress() const {
   assert((IsLoad() || opcode() == SpvOpStore || opcode() == SpvOpAccessChain ||
-          opcode() == SpvOpInBoundsAccessChain ||
-          opcode() == SpvOpCopyObject) &&
+          opcode() == SpvOpPtrAccessChain ||
+          opcode() == SpvOpInBoundsAccessChain || opcode() == SpvOpCopyObject ||
+          opcode() == SpvOpImageTexelPointer) &&
          "GetBaseAddress should only be called on instructions that take a "
          "pointer or image.");
   uint32_t base = GetSingleWordInOperand(kLoadBaseIndex);
-  ir::Instruction* base_inst = context()->get_def_use_mgr()->GetDef(base);
+  Instruction* base_inst = context()->get_def_use_mgr()->GetDef(base);
   bool done = false;
   while (!done) {
     switch (base_inst->opcode()) {
@@ -189,6 +188,8 @@ Instruction* Instruction::GetBaseAddress() const {
     case SpvOpStore:
     case SpvOpAccessChain:
     case SpvOpInBoundsAccessChain:
+    case SpvOpPtrAccessChain:
+    case SpvOpImageTexelPointer:
     case SpvOpCopyObject:
       // A load or store through a pointer.
       assert(base_inst->IsValidBasePointer() &&
@@ -219,7 +220,7 @@ bool Instruction::IsVulkanStorageImage() const {
     return false;
   }
 
-  ir::Instruction* base_type =
+  Instruction* base_type =
       context()->get_def_use_mgr()->GetDef(GetSingleWordInOperand(1));
   if (base_type->opcode() != SpvOpTypeImage) {
     return false;
@@ -245,7 +246,7 @@ bool Instruction::IsVulkanSampledImage() const {
     return false;
   }
 
-  ir::Instruction* base_type =
+  Instruction* base_type =
       context()->get_def_use_mgr()->GetDef(GetSingleWordInOperand(1));
   if (base_type->opcode() != SpvOpTypeImage) {
     return false;
@@ -271,7 +272,7 @@ bool Instruction::IsVulkanStorageTexelBuffer() const {
     return false;
   }
 
-  ir::Instruction* base_type =
+  Instruction* base_type =
       context()->get_def_use_mgr()->GetDef(GetSingleWordInOperand(1));
   if (base_type->opcode() != SpvOpTypeImage) {
     return false;
@@ -293,7 +294,7 @@ bool Instruction::IsVulkanStorageBuffer() const {
     return false;
   }
 
-  ir::Instruction* base_type =
+  Instruction* base_type =
       context()->get_def_use_mgr()->GetDef(GetSingleWordInOperand(1));
 
   if (base_type->opcode() != SpvOpTypeStruct) {
@@ -305,13 +306,13 @@ bool Instruction::IsVulkanStorageBuffer() const {
     bool is_buffer_block = false;
     context()->get_decoration_mgr()->ForEachDecoration(
         base_type->result_id(), SpvDecorationBufferBlock,
-        [&is_buffer_block](const ir::Instruction&) { is_buffer_block = true; });
+        [&is_buffer_block](const Instruction&) { is_buffer_block = true; });
     return is_buffer_block;
   } else if (storage_class == SpvStorageClassStorageBuffer) {
     bool is_block = false;
     context()->get_decoration_mgr()->ForEachDecoration(
         base_type->result_id(), SpvDecorationBlock,
-        [&is_block](const ir::Instruction&) { is_block = true; });
+        [&is_block](const Instruction&) { is_block = true; });
     return is_block;
   }
   return false;
@@ -327,7 +328,7 @@ bool Instruction::IsVulkanUniformBuffer() const {
     return false;
   }
 
-  ir::Instruction* base_type =
+  Instruction* base_type =
       context()->get_def_use_mgr()->GetDef(GetSingleWordInOperand(1));
   if (base_type->opcode() != SpvOpTypeStruct) {
     return false;
@@ -336,7 +337,7 @@ bool Instruction::IsVulkanUniformBuffer() const {
   bool is_block = false;
   context()->get_decoration_mgr()->ForEachDecoration(
       base_type->result_id(), SpvDecorationBlock,
-      [&is_block](const ir::Instruction&) { is_block = true; });
+      [&is_block](const Instruction&) { is_block = true; });
   return is_block;
 }
 
@@ -416,12 +417,13 @@ bool Instruction::IsValidBasePointer() const {
     return false;
   }
 
-  ir::Instruction* type = context()->get_def_use_mgr()->GetDef(tid);
+  Instruction* type = context()->get_def_use_mgr()->GetDef(tid);
   if (type->opcode() != SpvOpTypePointer) {
     return false;
   }
 
-  if (context()->get_feature_mgr()->HasCapability(SpvCapabilityAddresses)) {
+  auto feature_mgr = context()->get_feature_mgr();
+  if (feature_mgr->HasCapability(SpvCapabilityAddresses)) {
     // TODO: The rules here could be more restrictive.
     return true;
   }
@@ -430,8 +432,27 @@ bool Instruction::IsValidBasePointer() const {
     return true;
   }
 
+  // With variable pointers, there are more valid base pointer objects.
+  // Variable pointers implicitly declares Variable pointers storage buffer.
+  SpvStorageClass storage_class =
+      static_cast<SpvStorageClass>(type->GetSingleWordInOperand(0));
+  if ((feature_mgr->HasCapability(SpvCapabilityVariablePointersStorageBuffer) &&
+       storage_class == SpvStorageClassStorageBuffer) ||
+      (feature_mgr->HasCapability(SpvCapabilityVariablePointers) &&
+       storage_class == SpvStorageClassWorkgroup)) {
+    switch (opcode()) {
+      case SpvOpPhi:
+      case SpvOpSelect:
+      case SpvOpFunctionCall:
+      case SpvOpConstantNull:
+        return true;
+      default:
+        break;
+    }
+  }
+
   uint32_t pointee_type_id = type->GetSingleWordInOperand(1);
-  ir::Instruction* pointee_type_inst =
+  Instruction* pointee_type_inst =
       context()->get_def_use_mgr()->GetDef(pointee_type_id);
 
   if (pointee_type_inst->IsOpaqueType()) {
@@ -446,7 +467,7 @@ bool Instruction::IsValidBaseImage() const {
     return false;
   }
 
-  ir::Instruction* type = context()->get_def_use_mgr()->GetDef(tid);
+  Instruction* type = context()->get_def_use_mgr()->GetDef(tid);
   return (type->opcode() == SpvOpTypeImage ||
           type->opcode() == SpvOpTypeSampledImage);
 }
@@ -455,13 +476,13 @@ bool Instruction::IsOpaqueType() const {
   if (opcode() == SpvOpTypeStruct) {
     bool is_opaque = false;
     ForEachInOperand([&is_opaque, this](const uint32_t* op_id) {
-      ir::Instruction* type_inst = context()->get_def_use_mgr()->GetDef(*op_id);
+      Instruction* type_inst = context()->get_def_use_mgr()->GetDef(*op_id);
       is_opaque |= type_inst->IsOpaqueType();
     });
     return is_opaque;
   } else if (opcode() == SpvOpTypeArray) {
     uint32_t sub_type_id = GetSingleWordInOperand(0);
-    ir::Instruction* sub_type_inst =
+    Instruction* sub_type_inst =
         context()->get_def_use_mgr()->GetDef(sub_type_id);
     return sub_type_inst->IsOpaqueType();
   } else {
@@ -472,15 +493,16 @@ bool Instruction::IsOpaqueType() const {
 
 bool Instruction::IsFoldable() const {
   return IsFoldableByFoldScalar() ||
-         opt::GetConstantFoldingRules().HasFoldingRule(opcode());
+         context()->get_instruction_folder().HasConstFoldingRule(opcode());
 }
 
 bool Instruction::IsFoldableByFoldScalar() const {
-  if (!opt::IsFoldableOpcode(opcode())) {
+  const InstructionFolder& folder = context()->get_instruction_folder();
+  if (!folder.IsFoldableOpcode(opcode())) {
     return false;
   }
   Instruction* type = context()->get_def_use_mgr()->GetDef(type_id());
-  return opt::IsFoldableType(type);
+  return folder.IsFoldableType(type);
 }
 
 bool Instruction::IsFloatingPointFoldingAllowed() const {
@@ -491,8 +513,8 @@ bool Instruction::IsFloatingPointFoldingAllowed() const {
 
   bool is_nocontract = false;
   context_->get_decoration_mgr()->WhileEachDecoration(
-      opcode_, SpvDecorationNoContraction,
-      [&is_nocontract](const ir::Instruction&) {
+      result_id(), SpvDecorationNoContraction,
+      [&is_nocontract](const Instruction&) {
         is_nocontract = true;
         return false;
       });
@@ -516,16 +538,31 @@ std::string Instruction::PrettyPrint(uint32_t options) const {
       options | SPV_BINARY_TO_TEXT_OPTION_NO_HEADER);
 }
 
-std::ostream& operator<<(std::ostream& str, const ir::Instruction& inst) {
+std::ostream& operator<<(std::ostream& str, const Instruction& inst) {
   str << inst.PrettyPrint();
   return str;
 }
 
+void Instruction::Dump() const {
+  std::cerr << "Instruction #" << unique_id() << "\n" << *this << "\n";
+}
+
 bool Instruction::IsOpcodeCodeMotionSafe() const {
   switch (opcode_) {
+    case SpvOpNop:
+    case SpvOpUndef:
+    case SpvOpLoad:
+    case SpvOpAccessChain:
+    case SpvOpInBoundsAccessChain:
+    case SpvOpArrayLength:
     case SpvOpVectorExtractDynamic:
     case SpvOpVectorInsertDynamic:
     case SpvOpVectorShuffle:
+    case SpvOpCompositeConstruct:
+    case SpvOpCompositeExtract:
+    case SpvOpCompositeInsert:
+    case SpvOpCopyObject:
+    case SpvOpTranspose:
     case SpvOpConvertFToU:
     case SpvOpConvertFToS:
     case SpvOpConvertSToF:
@@ -556,11 +593,22 @@ bool Instruction::IsOpcodeCodeMotionSafe() const {
     case SpvOpVectorTimesMatrix:
     case SpvOpMatrixTimesVector:
     case SpvOpMatrixTimesMatrix:
+    case SpvOpOuterProduct:
+    case SpvOpDot:
+    case SpvOpIAddCarry:
+    case SpvOpISubBorrow:
+    case SpvOpUMulExtended:
+    case SpvOpSMulExtended:
+    case SpvOpAny:
+    case SpvOpAll:
+    case SpvOpIsNan:
+    case SpvOpIsInf:
     case SpvOpLogicalEqual:
     case SpvOpLogicalNotEqual:
     case SpvOpLogicalOr:
     case SpvOpLogicalAnd:
     case SpvOpLogicalNot:
+    case SpvOpSelect:
     case SpvOpIEqual:
     case SpvOpINotEqual:
     case SpvOpUGreaterThan:
@@ -590,11 +638,116 @@ bool Instruction::IsOpcodeCodeMotionSafe() const {
     case SpvOpBitwiseXor:
     case SpvOpBitwiseAnd:
     case SpvOpNot:
+    case SpvOpBitFieldInsert:
+    case SpvOpBitFieldSExtract:
+    case SpvOpBitFieldUExtract:
+    case SpvOpBitReverse:
+    case SpvOpBitCount:
+    case SpvOpSizeOf:
       return true;
     default:
       return false;
   }
 }
 
-}  // namespace ir
+bool Instruction::IsScalarizable() const {
+  if (spvOpcodeIsScalarizable(opcode())) {
+    return true;
+  }
+
+  const uint32_t kExtInstSetIdInIdx = 0;
+  const uint32_t kExtInstInstructionInIdx = 1;
+
+  if (opcode() == SpvOpExtInst) {
+    uint32_t instSetId =
+        context()->get_feature_mgr()->GetExtInstImportId_GLSLstd450();
+
+    if (GetSingleWordInOperand(kExtInstSetIdInIdx) == instSetId) {
+      switch (GetSingleWordInOperand(kExtInstInstructionInIdx)) {
+        case GLSLstd450Round:
+        case GLSLstd450RoundEven:
+        case GLSLstd450Trunc:
+        case GLSLstd450FAbs:
+        case GLSLstd450SAbs:
+        case GLSLstd450FSign:
+        case GLSLstd450SSign:
+        case GLSLstd450Floor:
+        case GLSLstd450Ceil:
+        case GLSLstd450Fract:
+        case GLSLstd450Radians:
+        case GLSLstd450Degrees:
+        case GLSLstd450Sin:
+        case GLSLstd450Cos:
+        case GLSLstd450Tan:
+        case GLSLstd450Asin:
+        case GLSLstd450Acos:
+        case GLSLstd450Atan:
+        case GLSLstd450Sinh:
+        case GLSLstd450Cosh:
+        case GLSLstd450Tanh:
+        case GLSLstd450Asinh:
+        case GLSLstd450Acosh:
+        case GLSLstd450Atanh:
+        case GLSLstd450Atan2:
+        case GLSLstd450Pow:
+        case GLSLstd450Exp:
+        case GLSLstd450Log:
+        case GLSLstd450Exp2:
+        case GLSLstd450Log2:
+        case GLSLstd450Sqrt:
+        case GLSLstd450InverseSqrt:
+        case GLSLstd450Modf:
+        case GLSLstd450FMin:
+        case GLSLstd450UMin:
+        case GLSLstd450SMin:
+        case GLSLstd450FMax:
+        case GLSLstd450UMax:
+        case GLSLstd450SMax:
+        case GLSLstd450FClamp:
+        case GLSLstd450UClamp:
+        case GLSLstd450SClamp:
+        case GLSLstd450FMix:
+        case GLSLstd450Step:
+        case GLSLstd450SmoothStep:
+        case GLSLstd450Fma:
+        case GLSLstd450Frexp:
+        case GLSLstd450Ldexp:
+        case GLSLstd450FindILsb:
+        case GLSLstd450FindSMsb:
+        case GLSLstd450FindUMsb:
+        case GLSLstd450NMin:
+        case GLSLstd450NMax:
+        case GLSLstd450NClamp:
+          return true;
+        default:
+          return false;
+      }
+    }
+  }
+  return false;
+}
+
+bool Instruction::IsOpcodeSafeToDelete() const {
+  if (context()->IsCombinatorInstruction(this)) {
+    return true;
+  }
+
+  switch (opcode()) {
+    case SpvOpDPdx:
+    case SpvOpDPdy:
+    case SpvOpFwidth:
+    case SpvOpDPdxFine:
+    case SpvOpDPdyFine:
+    case SpvOpFwidthFine:
+    case SpvOpDPdxCoarse:
+    case SpvOpDPdyCoarse:
+    case SpvOpFwidthCoarse:
+    case SpvOpImageQueryLod:
+      return true;
+    default:
+      return false;
+  }
+}
+
+}  // namespace opt
 }  // namespace spvtools
