@@ -12,41 +12,44 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "opt/loop_descriptor.h"
+#include "source/opt/loop_descriptor.h"
+
 #include <algorithm>
 #include <iostream>
+#include <limits>
+#include <stack>
 #include <type_traits>
 #include <utility>
 #include <vector>
 
-#include "constants.h"
-#include "opt/cfg.h"
-#include "opt/dominator_tree.h"
-#include "opt/ir_builder.h"
-#include "opt/ir_context.h"
-#include "opt/iterator.h"
-#include "opt/make_unique.h"
-#include "opt/tree_iterator.h"
+#include "source/opt/cfg.h"
+#include "source/opt/constants.h"
+#include "source/opt/dominator_tree.h"
+#include "source/opt/ir_builder.h"
+#include "source/opt/ir_context.h"
+#include "source/opt/iterator.h"
+#include "source/opt/tree_iterator.h"
+#include "source/util/make_unique.h"
 
 namespace spvtools {
-namespace ir {
+namespace opt {
 
 // Takes in a phi instruction |induction| and the loop |header| and returns the
 // step operation of the loop.
-ir::Instruction* Loop::GetInductionStepOperation(
-    const ir::Instruction* induction) const {
+Instruction* Loop::GetInductionStepOperation(
+    const Instruction* induction) const {
   // Induction must be a phi instruction.
   assert(induction->opcode() == SpvOpPhi);
 
-  ir::Instruction* step = nullptr;
+  Instruction* step = nullptr;
 
-  opt::analysis::DefUseManager* def_use_manager = context_->get_def_use_mgr();
+  analysis::DefUseManager* def_use_manager = context_->get_def_use_mgr();
 
   // Traverse the incoming operands of the phi instruction.
   for (uint32_t operand_id = 1; operand_id < induction->NumInOperands();
        operand_id += 2) {
     // Incoming edge.
-    ir::BasicBlock* incoming_block =
+    BasicBlock* incoming_block =
         context_->cfg()->block(induction->GetSingleWordInOperand(operand_id));
 
     // Check if the block is dominated by header, and thus coming from within
@@ -142,17 +145,36 @@ int64_t Loop::GetResidualConditionValue(SpvOp condition, int64_t initial_value,
   return remainder;
 }
 
+Instruction* Loop::GetConditionInst() const {
+  BasicBlock* condition_block = FindConditionBlock();
+  if (!condition_block) {
+    return nullptr;
+  }
+  Instruction* branch_conditional = &*condition_block->tail();
+  if (!branch_conditional ||
+      branch_conditional->opcode() != SpvOpBranchConditional) {
+    return nullptr;
+  }
+  Instruction* condition_inst = context_->get_def_use_mgr()->GetDef(
+      branch_conditional->GetSingleWordInOperand(0));
+  if (IsSupportedCondition(condition_inst->opcode())) {
+    return condition_inst;
+  }
+
+  return nullptr;
+}
+
 // Extract the initial value from the |induction| OpPhi instruction and store it
 // in |value|. If the function couldn't find the initial value of |induction|
 // return false.
-bool Loop::GetInductionInitValue(const ir::Instruction* induction,
+bool Loop::GetInductionInitValue(const Instruction* induction,
                                  int64_t* value) const {
-  ir::Instruction* constant_instruction = nullptr;
-  opt::analysis::DefUseManager* def_use_manager = context_->get_def_use_mgr();
+  Instruction* constant_instruction = nullptr;
+  analysis::DefUseManager* def_use_manager = context_->get_def_use_mgr();
 
   for (uint32_t operand_id = 0; operand_id < induction->NumInOperands();
        operand_id += 2) {
-    ir::BasicBlock* bb = context_->cfg()->block(
+    BasicBlock* bb = context_->cfg()->block(
         induction->GetSingleWordInOperand(operand_id + 1));
 
     if (!IsInsideLoop(bb)) {
@@ -163,13 +185,13 @@ bool Loop::GetInductionInitValue(const ir::Instruction* induction,
 
   if (!constant_instruction) return false;
 
-  const opt::analysis::Constant* constant =
+  const analysis::Constant* constant =
       context_->get_constant_mgr()->FindDeclaredConstant(
           constant_instruction->result_id());
   if (!constant) return false;
 
   if (value) {
-    const opt::analysis::Integer* type =
+    const analysis::Integer* type =
         constant->AsIntConstant()->type()->AsInteger();
 
     if (type->IsSigned()) {
@@ -182,7 +204,7 @@ bool Loop::GetInductionInitValue(const ir::Instruction* induction,
   return true;
 }
 
-Loop::Loop(IRContext* context, opt::DominatorAnalysis* dom_analysis,
+Loop::Loop(IRContext* context, DominatorAnalysis* dom_analysis,
            BasicBlock* header, BasicBlock* continue_target,
            BasicBlock* merge_target)
     : context_(context),
@@ -195,19 +217,20 @@ Loop::Loop(IRContext* context, opt::DominatorAnalysis* dom_analysis,
   assert(context);
   assert(dom_analysis);
   loop_preheader_ = FindLoopPreheader(dom_analysis);
+  loop_latch_ = FindLatchBlock();
 }
 
-BasicBlock* Loop::FindLoopPreheader(opt::DominatorAnalysis* dom_analysis) {
+BasicBlock* Loop::FindLoopPreheader(DominatorAnalysis* dom_analysis) {
   CFG* cfg = context_->cfg();
-  opt::DominatorTree& dom_tree = dom_analysis->GetDomTree();
-  opt::DominatorTreeNode* header_node = dom_tree.GetTreeNode(loop_header_);
+  DominatorTree& dom_tree = dom_analysis->GetDomTree();
+  DominatorTreeNode* header_node = dom_tree.GetTreeNode(loop_header_);
 
   // The loop predecessor.
   BasicBlock* loop_pred = nullptr;
 
   auto header_pred = cfg->preds(loop_header_->id());
   for (uint32_t p_id : header_pred) {
-    opt::DominatorTreeNode* node = dom_tree.GetTreeNode(p_id);
+    DominatorTreeNode* node = dom_tree.GetTreeNode(p_id);
     if (node && !dom_tree.Dominates(header_node, node)) {
       // The predecessor is not part of the loop, so potential loop preheader.
       if (loop_pred && node->bb_ != loop_pred) {
@@ -244,8 +267,8 @@ bool Loop::IsInsideLoop(Instruction* inst) const {
 
 bool Loop::IsBasicBlockInLoopSlow(const BasicBlock* bb) {
   assert(bb->GetParent() && "The basic block does not belong to a function");
-  opt::DominatorAnalysis* dom_analysis =
-      context_->GetDominatorAnalysis(bb->GetParent(), *context_->cfg());
+  DominatorAnalysis* dom_analysis =
+      context_->GetDominatorAnalysis(bb->GetParent());
   if (dom_analysis->IsReachable(bb) &&
       !dom_analysis->Dominates(GetHeaderBlock(), bb))
     return false;
@@ -256,101 +279,16 @@ bool Loop::IsBasicBlockInLoopSlow(const BasicBlock* bb) {
 BasicBlock* Loop::GetOrCreatePreHeaderBlock() {
   if (loop_preheader_) return loop_preheader_;
 
-  Function* fn = loop_header_->GetParent();
-  // Find the insertion point for the preheader.
-  Function::iterator header_it =
-      std::find_if(fn->begin(), fn->end(),
-                   [this](BasicBlock& bb) { return &bb == loop_header_; });
-  assert(header_it != fn->end());
-
-  // Create the preheader basic block.
-  loop_preheader_ = &*header_it.InsertBefore(std::unique_ptr<ir::BasicBlock>(
-      new ir::BasicBlock(std::unique_ptr<ir::Instruction>(new ir::Instruction(
-          context_, SpvOpLabel, 0, context_->TakeNextId(), {})))));
-  loop_preheader_->SetParent(fn);
-  uint32_t loop_preheader_id = loop_preheader_->id();
-
-  // Redirect the branches and patch the phi:
-  //  - For each phi instruction in the header:
-  //    - If the header has only 1 out-of-loop incoming branch:
-  //      - Change the incomning branch to be the preheader.
-  //    - If the header has more than 1 out-of-loop incoming branch:
-  //      - Create a new phi in the preheader, gathering all out-of-loops
-  //      incoming values;
-  //      - Patch the header phi instruction to use the preheader phi
-  //      instruction;
-  //  - Redirect all edges coming from outside the loop to the preheader.
-  opt::InstructionBuilder builder(
-      context_, loop_preheader_,
-      ir::IRContext::kAnalysisDefUse |
-          ir::IRContext::kAnalysisInstrToBlockMapping);
-  // Patch all the phi instructions.
-  loop_header_->ForEachPhiInst([&builder, this](Instruction* phi) {
-    std::vector<uint32_t> preheader_phi_ops;
-    std::vector<uint32_t> header_phi_ops;
-    for (uint32_t i = 0; i < phi->NumInOperands(); i += 2) {
-      uint32_t def_id = phi->GetSingleWordInOperand(i);
-      uint32_t branch_id = phi->GetSingleWordInOperand(i + 1);
-      if (IsInsideLoop(branch_id)) {
-        header_phi_ops.push_back(def_id);
-        header_phi_ops.push_back(branch_id);
-      } else {
-        preheader_phi_ops.push_back(def_id);
-        preheader_phi_ops.push_back(branch_id);
-      }
-    }
-
-    Instruction* preheader_insn_def = nullptr;
-    // Create a phi instruction if and only if the preheader_phi_ops has more
-    // than one pair.
-    if (preheader_phi_ops.size() > 2)
-      preheader_insn_def = builder.AddPhi(phi->type_id(), preheader_phi_ops);
-    else
-      preheader_insn_def =
-          context_->get_def_use_mgr()->GetDef(preheader_phi_ops[0]);
-    // Build the new incoming edge.
-    header_phi_ops.push_back(preheader_insn_def->result_id());
-    header_phi_ops.push_back(loop_preheader_->id());
-    // Rewrite operands of the header's phi instruction.
-    uint32_t idx = 0;
-    for (; idx < header_phi_ops.size(); idx++)
-      phi->SetInOperand(idx, {header_phi_ops[idx]});
-    // Remove extra operands, from last to first (more efficient).
-    for (uint32_t j = phi->NumInOperands() - 1; j >= idx; j--)
-      phi->RemoveInOperand(j);
-  });
-  // Branch from the preheader to the header.
-  builder.AddBranch(loop_header_->id());
-
-  // Redirect all out of loop branches to the header to the preheader.
   CFG* cfg = context_->cfg();
-  cfg->RegisterBlock(loop_preheader_);
-  for (uint32_t pred_id : cfg->preds(loop_header_->id())) {
-    if (pred_id == loop_preheader_->id()) continue;
-    if (IsInsideLoop(pred_id)) continue;
-    BasicBlock* pred = cfg->block(pred_id);
-    pred->ForEachSuccessorLabel([this, loop_preheader_id](uint32_t* id) {
-      if (*id == loop_header_->id()) *id = loop_preheader_id;
-    });
-    cfg->AddEdge(pred_id, loop_preheader_id);
-  }
-  // Delete predecessors that are no longer predecessors of the loop header.
-  cfg->RemoveNonExistingEdges(loop_header_->id());
-  // Update the loop descriptors.
-  if (HasParent()) {
-    GetParent()->AddBasicBlock(loop_preheader_);
-    context_->GetLoopDescriptor(fn)->SetBasicBlockToLoop(loop_preheader_->id(),
-                                                         GetParent());
-  }
-
-  context_->InvalidateAnalysesExceptFor(
-      builder.GetPreservedAnalysis() |
-      ir::IRContext::Analysis::kAnalysisLoopAnalysis |
-      ir::IRContext::kAnalysisCFG);
-
+  loop_header_ = cfg->SplitLoopHeader(loop_header_);
   return loop_preheader_;
 }
 
+void Loop::SetContinueBlock(BasicBlock* continue_block) {
+  assert(IsInsideLoop(continue_block));
+  loop_continue_ = continue_block;
+}
+
 void Loop::SetLatchBlock(BasicBlock* latch) {
 #ifndef NDEBUG
   assert(latch->GetParent() && "The basic block does not belong to a function");
@@ -379,22 +317,47 @@ void Loop::SetMergeBlock(BasicBlock* merge) {
 }
 
 void Loop::SetPreHeaderBlock(BasicBlock* preheader) {
-  assert(!IsInsideLoop(preheader) && "The preheader block is in the loop");
-  assert(preheader->tail()->opcode() == SpvOpBranch &&
-         "The preheader block does not unconditionally branch to the header "
-         "block");
-  assert(preheader->tail()->GetSingleWordOperand(0) == GetHeaderBlock()->id() &&
-         "The preheader block does not unconditionally branch to the header "
-         "block");
+  if (preheader) {
+    assert(!IsInsideLoop(preheader) && "The preheader block is in the loop");
+    assert(preheader->tail()->opcode() == SpvOpBranch &&
+           "The preheader block does not unconditionally branch to the header "
+           "block");
+    assert(preheader->tail()->GetSingleWordOperand(0) ==
+               GetHeaderBlock()->id() &&
+           "The preheader block does not unconditionally branch to the header "
+           "block");
+  }
   loop_preheader_ = preheader;
 }
 
+BasicBlock* Loop::FindLatchBlock() {
+  CFG* cfg = context_->cfg();
+
+  DominatorAnalysis* dominator_analysis =
+      context_->GetDominatorAnalysis(loop_header_->GetParent());
+
+  // Look at the predecessors of the loop header to find a predecessor block
+  // which is dominated by the loop continue target. There should only be one
+  // block which meets this criteria and this is the latch block, as per the
+  // SPIR-V spec.
+  for (uint32_t block_id : cfg->preds(loop_header_->id())) {
+    if (dominator_analysis->Dominates(loop_continue_->id(), block_id)) {
+      return cfg->block(block_id);
+    }
+  }
+
+  assert(
+      false &&
+      "Every loop should have a latch block dominated by the continue target");
+  return nullptr;
+}
+
 void Loop::GetExitBlocks(std::unordered_set<uint32_t>* exit_blocks) const {
-  ir::CFG* cfg = context_->cfg();
+  CFG* cfg = context_->cfg();
   exit_blocks->clear();
 
   for (uint32_t bb_id : GetBlocks()) {
-    const spvtools::ir::BasicBlock* bb = cfg->block(bb_id);
+    const BasicBlock* bb = cfg->block(bb_id);
     bb->ForEachSuccessorLabel([exit_blocks, this](uint32_t succ) {
       if (!IsInsideLoop(succ)) {
         exit_blocks->insert(succ);
@@ -406,13 +369,13 @@ void Loop::GetExitBlocks(std::unordered_set<uint32_t>* exit_blocks) const {
 void Loop::GetMergingBlocks(
     std::unordered_set<uint32_t>* merging_blocks) const {
   assert(GetMergeBlock() && "This loop is not structured");
-  ir::CFG* cfg = context_->cfg();
+  CFG* cfg = context_->cfg();
   merging_blocks->clear();
 
-  std::stack<const ir::BasicBlock*> to_visit;
+  std::stack<const BasicBlock*> to_visit;
   to_visit.push(GetMergeBlock());
   while (!to_visit.empty()) {
-    const ir::BasicBlock* bb = to_visit.top();
+    const BasicBlock* bb = to_visit.top();
     to_visit.pop();
     merging_blocks->insert(bb->id());
     for (uint32_t pred_id : cfg->preds(bb->id())) {
@@ -426,7 +389,7 @@ void Loop::GetMergingBlocks(
 namespace {
 
 static inline bool IsBasicBlockSafeToClone(IRContext* context, BasicBlock* bb) {
-  for (ir::Instruction& inst : *bb) {
+  for (Instruction& inst : *bb) {
     if (!inst.IsBranch() && !context->IsCombinatorInstruction(&inst))
       return false;
   }
@@ -437,7 +400,7 @@ static inline bool IsBasicBlockSafeToClone(IRContext* context, BasicBlock* bb) {
 }  // namespace
 
 bool Loop::IsSafeToClone() const {
-  ir::CFG& cfg = *context_->cfg();
+  CFG& cfg = *context_->cfg();
 
   for (uint32_t bb_id : GetBlocks()) {
     BasicBlock* bb = cfg.block(bb_id);
@@ -461,14 +424,14 @@ bool Loop::IsSafeToClone() const {
 }
 
 bool Loop::IsLCSSA() const {
-  ir::CFG* cfg = context_->cfg();
-  opt::analysis::DefUseManager* def_use_mgr = context_->get_def_use_mgr();
+  CFG* cfg = context_->cfg();
+  analysis::DefUseManager* def_use_mgr = context_->get_def_use_mgr();
 
   std::unordered_set<uint32_t> exit_blocks;
   GetExitBlocks(&exit_blocks);
 
   // Declare ir_context so we can capture context_ in the below lambda
-  ir::IRContext* ir_context = context_;
+  IRContext* ir_context = context_;
 
   for (uint32_t bb_id : GetBlocks()) {
     for (Instruction& insn : *cfg->block(bb_id)) {
@@ -477,7 +440,7 @@ bool Loop::IsLCSSA() const {
       //  - In an exit block and in a phi instruction.
       if (!def_use_mgr->WhileEachUser(
               &insn,
-              [&exit_blocks, ir_context, this](ir::Instruction* use) -> bool {
+              [&exit_blocks, ir_context, this](Instruction* use) -> bool {
                 BasicBlock* parent = ir_context->get_instr_block(use);
                 assert(parent && "Invalid analysis");
                 if (IsInsideLoop(parent)) return true;
@@ -496,7 +459,7 @@ bool Loop::ShouldHoistInstruction(IRContext* context, Instruction* inst) {
 }
 
 bool Loop::AreAllOperandsOutsideLoop(IRContext* context, Instruction* inst) {
-  opt::analysis::DefUseManager* def_use_mgr = context->get_def_use_mgr();
+  analysis::DefUseManager* def_use_mgr = context->get_def_use_mgr();
   bool all_outside_loop = true;
 
   const std::function<void(uint32_t*)> operand_outside_loop =
@@ -512,9 +475,9 @@ bool Loop::AreAllOperandsOutsideLoop(IRContext* context, Instruction* inst) {
 }
 
 void Loop::ComputeLoopStructuredOrder(
-    std::vector<ir::BasicBlock*>* ordered_loop_blocks, bool include_pre_header,
+    std::vector<BasicBlock*>* ordered_loop_blocks, bool include_pre_header,
     bool include_merge) const {
-  ir::CFG& cfg = *context_->cfg();
+  CFG& cfg = *context_->cfg();
 
   // Reserve the memory: all blocks in the loop + extra if needed.
   ordered_loop_blocks->reserve(GetBlocks().size() + include_pre_header +
@@ -530,26 +493,23 @@ void Loop::ComputeLoopStructuredOrder(
     ordered_loop_blocks->push_back(loop_merge_);
 }
 
-LoopDescriptor::LoopDescriptor(const Function* f)
+LoopDescriptor::LoopDescriptor(IRContext* context, const Function* f)
     : loops_(), dummy_top_loop_(nullptr) {
-  PopulateList(f);
+  PopulateList(context, f);
 }
 
 LoopDescriptor::~LoopDescriptor() { ClearLoops(); }
 
-void LoopDescriptor::PopulateList(const Function* f) {
-  IRContext* context = f->GetParent()->context();
-
-  opt::DominatorAnalysis* dom_analysis =
-      context->GetDominatorAnalysis(f, *context->cfg());
+void LoopDescriptor::PopulateList(IRContext* context, const Function* f) {
+  DominatorAnalysis* dom_analysis = context->GetDominatorAnalysis(f);
 
   ClearLoops();
 
   // Post-order traversal of the dominator tree to find all the OpLoopMerge
   // instructions.
-  opt::DominatorTree& dom_tree = dom_analysis->GetDomTree();
-  for (opt::DominatorTreeNode& node :
-       ir::make_range(dom_tree.post_begin(), dom_tree.post_end())) {
+  DominatorTree& dom_tree = dom_analysis->GetDomTree();
+  for (DominatorTreeNode& node :
+       make_range(dom_tree.post_begin(), dom_tree.post_end())) {
     Instruction* merge_inst = node.bb_->GetLoopMergeInst();
     if (merge_inst) {
       bool all_backedge_unreachable = true;
@@ -603,8 +563,8 @@ void LoopDescriptor::PopulateList(const Function* f) {
 
         current_loop->AddNestedLoop(previous_loop);
       }
-      opt::DominatorTreeNode* dom_merge_node = dom_tree.GetTreeNode(merge_bb);
-      for (opt::DominatorTreeNode& loop_node :
+      DominatorTreeNode* dom_merge_node = dom_tree.GetTreeNode(merge_bb);
+      for (DominatorTreeNode& loop_node :
            make_range(node.df_begin(), node.df_end())) {
         // Check if we are in the loop.
         if (dom_tree.Dominates(dom_merge_node, &loop_node)) continue;
@@ -619,17 +579,55 @@ void LoopDescriptor::PopulateList(const Function* f) {
   }
 }
 
-ir::BasicBlock* Loop::FindConditionBlock() const {
-  const ir::Function& function = *loop_merge_->GetParent();
-  ir::BasicBlock* condition_block = nullptr;
+std::vector<Loop*> LoopDescriptor::GetLoopsInBinaryLayoutOrder() {
+  std::vector<uint32_t> ids{};
 
-  const opt::DominatorAnalysis* dom_analysis =
-      context_->GetDominatorAnalysis(&function, *context_->cfg());
-  ir::BasicBlock* bb = dom_analysis->ImmediateDominator(loop_merge_);
+  for (size_t i = 0; i < NumLoops(); ++i) {
+    ids.push_back(GetLoopByIndex(i).GetHeaderBlock()->id());
+  }
+
+  std::vector<Loop*> loops{};
+  if (!ids.empty()) {
+    auto function = GetLoopByIndex(0).GetHeaderBlock()->GetParent();
+    for (const auto& block : *function) {
+      auto block_id = block.id();
+
+      auto element = std::find(std::begin(ids), std::end(ids), block_id);
+      if (element != std::end(ids)) {
+        loops.push_back(&GetLoopByIndex(element - std::begin(ids)));
+      }
+    }
+  }
+
+  return loops;
+}
+
+BasicBlock* Loop::FindConditionBlock() const {
+  if (!loop_merge_) {
+    return nullptr;
+  }
+  BasicBlock* condition_block = nullptr;
+
+  uint32_t in_loop_pred = 0;
+  for (uint32_t p : context_->cfg()->preds(loop_merge_->id())) {
+    if (IsInsideLoop(p)) {
+      if (in_loop_pred) {
+        // 2 in-loop predecessors.
+        return nullptr;
+      }
+      in_loop_pred = p;
+    }
+  }
+  if (!in_loop_pred) {
+    // Merge block is unreachable.
+    return nullptr;
+  }
+
+  BasicBlock* bb = context_->cfg()->block(in_loop_pred);
 
   if (!bb) return nullptr;
 
-  const ir::Instruction& branch = *bb->ctail();
+  const Instruction& branch = *bb->ctail();
 
   // Make sure the branch is a conditional branch.
   if (branch.opcode() != SpvOpBranchConditional) return nullptr;
@@ -643,35 +641,39 @@ ir::BasicBlock* Loop::FindConditionBlock() const {
   return condition_block;
 }
 
-bool Loop::FindNumberOfIterations(const ir::Instruction* induction,
-                                  const ir::Instruction* branch_inst,
+bool Loop::FindNumberOfIterations(const Instruction* induction,
+                                  const Instruction* branch_inst,
                                   size_t* iterations_out,
                                   int64_t* step_value_out,
                                   int64_t* init_value_out) const {
   // From the branch instruction find the branch condition.
-  opt::analysis::DefUseManager* def_use_manager = context_->get_def_use_mgr();
+  analysis::DefUseManager* def_use_manager = context_->get_def_use_mgr();
 
   // Condition instruction from the OpConditionalBranch.
-  ir::Instruction* condition =
+  Instruction* condition =
       def_use_manager->GetDef(branch_inst->GetSingleWordOperand(0));
 
   assert(IsSupportedCondition(condition->opcode()));
 
   // Get the constant manager from the ir context.
-  opt::analysis::ConstantManager* const_manager = context_->get_constant_mgr();
+  analysis::ConstantManager* const_manager = context_->get_constant_mgr();
 
   // Find the constant value used by the condition variable. Exit out if it
   // isn't a constant int.
-  const opt::analysis::Constant* upper_bound =
+  const analysis::Constant* upper_bound =
       const_manager->FindDeclaredConstant(condition->GetSingleWordOperand(3));
   if (!upper_bound) return false;
 
   // Must be integer because of the opcode on the condition.
   int64_t condition_value = 0;
 
-  const opt::analysis::Integer* type =
+  const analysis::Integer* type =
       upper_bound->AsIntConstant()->type()->AsInteger();
 
+  if (type->width() > 32) {
+    return false;
+  }
+
   if (type->IsSigned()) {
     condition_value = upper_bound->AsIntConstant()->GetS32BitValue();
   } else {
@@ -679,18 +681,18 @@ bool Loop::FindNumberOfIterations(const ir::Instruction* induction,
   }
 
   // Find the instruction which is stepping through the loop.
-  ir::Instruction* step_inst = GetInductionStepOperation(induction);
+  Instruction* step_inst = GetInductionStepOperation(induction);
   if (!step_inst) return false;
 
   // Find the constant value used by the condition variable.
-  const opt::analysis::Constant* step_constant =
+  const analysis::Constant* step_constant =
       const_manager->FindDeclaredConstant(step_inst->GetSingleWordOperand(3));
   if (!step_constant) return false;
 
   // Must be integer because of the opcode on the condition.
   int64_t step_value = 0;
 
-  const opt::analysis::Integer* step_type =
+  const analysis::Integer* step_type =
       step_constant->AsIntConstant()->type()->AsInteger();
 
   if (step_type->IsSigned()) {
@@ -832,34 +834,34 @@ int64_t Loop::GetIterations(SpvOp condition, int64_t condition_value,
 
 // Returns the list of induction variables within the loop.
 void Loop::GetInductionVariables(
-    std::vector<ir::Instruction*>& induction_variables) const {
-  for (ir::Instruction& inst : *loop_header_) {
+    std::vector<Instruction*>& induction_variables) const {
+  for (Instruction& inst : *loop_header_) {
     if (inst.opcode() == SpvOp::SpvOpPhi) {
       induction_variables.push_back(&inst);
     }
   }
 }
 
-ir::Instruction* Loop::FindConditionVariable(
-    const ir::BasicBlock* condition_block) const {
+Instruction* Loop::FindConditionVariable(
+    const BasicBlock* condition_block) const {
   // Find the branch instruction.
-  const ir::Instruction& branch_inst = *condition_block->ctail();
+  const Instruction& branch_inst = *condition_block->ctail();
 
-  ir::Instruction* induction = nullptr;
+  Instruction* induction = nullptr;
   // Verify that the branch instruction is a conditional branch.
   if (branch_inst.opcode() == SpvOp::SpvOpBranchConditional) {
     // From the branch instruction find the branch condition.
-    opt::analysis::DefUseManager* def_use_manager = context_->get_def_use_mgr();
+    analysis::DefUseManager* def_use_manager = context_->get_def_use_mgr();
 
     // Find the instruction representing the condition used in the conditional
     // branch.
-    ir::Instruction* condition =
+    Instruction* condition =
         def_use_manager->GetDef(branch_inst.GetSingleWordOperand(0));
 
     // Ensure that the condition is a less than operation.
     if (condition && IsSupportedCondition(condition->opcode())) {
       // The left hand side operand of the operation.
-      ir::Instruction* variable_inst =
+      Instruction* variable_inst =
           def_use_manager->GetDef(condition->GetSingleWordOperand(2));
 
       // Make sure the variable instruction used is a phi.
@@ -879,18 +881,18 @@ ir::Instruction* Loop::FindConditionVariable(
         uint32_t operand_label_2 = 3;
 
         // Make sure one of them is the preheader.
-        if (variable_inst->GetSingleWordInOperand(operand_label_1) !=
-                loop_preheader_->id() &&
-            variable_inst->GetSingleWordInOperand(operand_label_2) !=
-                loop_preheader_->id()) {
+        if (!IsInsideLoop(
+                variable_inst->GetSingleWordInOperand(operand_label_1)) &&
+            !IsInsideLoop(
+                variable_inst->GetSingleWordInOperand(operand_label_2))) {
           return nullptr;
         }
 
         // And make sure that the other is the latch block.
         if (variable_inst->GetSingleWordInOperand(operand_label_1) !=
-                loop_continue_->id() &&
+                loop_latch_->id() &&
             variable_inst->GetSingleWordInOperand(operand_label_2) !=
-                loop_continue_->id()) {
+                loop_latch_->id()) {
           return nullptr;
         }
       } else {
@@ -906,11 +908,25 @@ ir::Instruction* Loop::FindConditionVariable(
   return induction;
 }
 
+bool LoopDescriptor::CreatePreHeaderBlocksIfMissing() {
+  auto modified = false;
+
+  for (auto& loop : *this) {
+    if (!loop.GetPreHeaderBlock()) {
+      modified = true;
+      // TODO(1841): Handle failure to create pre-header.
+      loop.GetOrCreatePreHeaderBlock();
+    }
+  }
+
+  return modified;
+}
+
 // Add and remove loops which have been marked for addition and removal to
 // maintain the state of the loop descriptor class.
 void LoopDescriptor::PostModificationCleanup() {
   LoopContainerType loops_to_remove_;
-  for (ir::Loop* loop : loops_) {
+  for (Loop* loop : loops_) {
     if (loop->IsMarkedForRemoval()) {
       loops_to_remove_.push_back(loop);
       if (loop->HasParent()) {
@@ -919,13 +935,13 @@ void LoopDescriptor::PostModificationCleanup() {
     }
   }
 
-  for (ir::Loop* loop : loops_to_remove_) {
+  for (Loop* loop : loops_to_remove_) {
     loops_.erase(std::find(loops_.begin(), loops_.end(), loop));
   }
 
   for (auto& pair : loops_to_add_) {
-    ir::Loop* parent = pair.first;
-    ir::Loop* loop = pair.second;
+    Loop* parent = pair.first;
+    Loop* loop = pair.second;
 
     if (parent) {
       loop->SetParent(nullptr);
@@ -950,12 +966,12 @@ void LoopDescriptor::ClearLoops() {
 }
 
 // Adds a new loop nest to the descriptor set.
-ir::Loop* LoopDescriptor::AddLoopNest(std::unique_ptr<ir::Loop> new_loop) {
-  ir::Loop* loop = new_loop.release();
+Loop* LoopDescriptor::AddLoopNest(std::unique_ptr<Loop> new_loop) {
+  Loop* loop = new_loop.release();
   if (!loop->HasParent()) dummy_top_loop_.nested_loops_.push_back(loop);
   // Iterate from inner to outer most loop, adding basic block to loop mapping
   // as we go.
-  for (ir::Loop& current_loop :
+  for (Loop& current_loop :
        make_range(iterator::begin(loop), iterator::end(nullptr))) {
     loops_.push_back(&current_loop);
     for (uint32_t bb_id : current_loop.GetBlocks())
@@ -965,18 +981,18 @@ ir::Loop* LoopDescriptor::AddLoopNest(std::unique_ptr<ir::Loop> new_loop) {
   return loop;
 }
 
-void LoopDescriptor::RemoveLoop(ir::Loop* loop) {
-  ir::Loop* parent = loop->GetParent() ? loop->GetParent() : &dummy_top_loop_;
+void LoopDescriptor::RemoveLoop(Loop* loop) {
+  Loop* parent = loop->GetParent() ? loop->GetParent() : &dummy_top_loop_;
   parent->nested_loops_.erase(std::find(parent->nested_loops_.begin(),
                                         parent->nested_loops_.end(), loop));
   std::for_each(
       loop->nested_loops_.begin(), loop->nested_loops_.end(),
-      [loop](ir::Loop* sub_loop) { sub_loop->SetParent(loop->GetParent()); });
+      [loop](Loop* sub_loop) { sub_loop->SetParent(loop->GetParent()); });
   parent->nested_loops_.insert(parent->nested_loops_.end(),
                                loop->nested_loops_.begin(),
                                loop->nested_loops_.end());
   for (uint32_t bb_id : loop->GetBlocks()) {
-    ir::Loop* l = FindLoopForBasicBlock(bb_id);
+    Loop* l = FindLoopForBasicBlock(bb_id);
     if (l == loop) {
       SetBasicBlockToLoop(bb_id, l->GetParent());
     } else {
@@ -991,5 +1007,5 @@ void LoopDescriptor::RemoveLoop(ir::Loop* loop) {
   loops_.erase(it);
 }
 
-}  // namespace ir
+}  // namespace opt
 }  // namespace spvtools
