@@ -12,11 +12,13 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "unit_spirv.h"
+#include <string>
+#include <vector>
 
-#include <gmock/gmock.h>
 #include "DebugInfo.h"
-#include "test_fixture.h"
+#include "gmock/gmock.h"
+#include "test/test_fixture.h"
+#include "test/unit_spirv.h"
 
 // This file tests the correctness of encoding and decoding of instructions
 // involving the DebugInfo extended instruction set.
@@ -24,6 +26,7 @@
 //
 // See https://www.khronos.org/registry/spir-v/specs/1.0/DebugInfo.html
 
+namespace spvtools {
 namespace {
 
 using spvtest::Concatenate;
@@ -805,4 +808,5 @@ INSTANTIATE_TEST_CASE_P(DebugInfoDebugMacroUndef, ExtInstDebugInfoRoundTripTest,
 #undef CASE_EL
 #undef CASE_ELL
 
-}  // anonymous namespace
+}  // namespace
+}  // namespace spvtools
