@@ -12,7 +12,6 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <spirv_validator_options.h>
 #include <algorithm>
 #include <cassert>
 #include <cstring>
@@ -20,15 +19,16 @@
 #include <iostream>
 #include <memory>
 #include <sstream>
+#include <string>
 #include <vector>
 
-#include "opt/set_spec_constant_default_value_pass.h"
+#include "source/opt/log.h"
+#include "source/spirv_target_env.h"
+#include "source/util/string_utils.h"
+#include "spirv-tools/libspirv.hpp"
 #include "spirv-tools/optimizer.hpp"
-
-#include "message.h"
 #include "tools/io.h"
-
-using namespace spvtools;
+#include "tools/util/cli_consumer.h"
 
 namespace {
 
@@ -40,6 +40,17 @@ struct OptStatus {
   int code;
 };
 
+// Message consumer for this tool.  Used to emit diagnostics during
+// initialization and setup. Note that |source| and |position| are irrelevant
+// here because we are still not processing a SPIR-V input file.
+void opt_diagnostic(spv_message_level_t level, const char* /*source*/,
+                    const spv_position_t& /*positon*/, const char* message) {
+  if (level == SPV_MSG_ERROR) {
+    fprintf(stderr, "error: ");
+  }
+  fprintf(stderr, "%s\n", message);
+}
+
 std::string GetListOfPassesAsString(const spvtools::Optimizer& optimizer) {
   std::stringstream ss;
   for (const auto& name : optimizer.GetPassNames()) {
@@ -68,6 +79,12 @@ std::string GetSizePasses() {
   return GetListOfPassesAsString(optimizer);
 }
 
+std::string GetWebGPUPasses() {
+  spvtools::Optimizer optimizer(SPV_ENV_WEBGPU_0);
+  optimizer.RegisterWebGPUPasses();
+  return GetListOfPassesAsString(optimizer);
+}
+
 void PrintUsage(const char* program) {
   // NOTE: Please maintain flags in lexicographical order.
   printf(
@@ -93,6 +110,9 @@ Options (in lexicographical order):
                Cleanup the control flow graph. This will remove any unnecessary
                code from the CFG like unreachable code. Performed on entry
                point call tree functions and exported functions.
+  --combine-access-chains
+               Combines chained access chains to produce a single instruction
+               where possible.
   --compact-ids
                Remap result ids to a compact range starting from %%1 and without
                any gaps.
@@ -102,6 +122,10 @@ Options (in lexicographical order):
                on function scope variables referenced only with load, store,
                and constant index access chains in entry point call tree
                functions.
+  --copy-propagate-arrays
+               Does propagation of memory references when an array is a copy of
+               another.  It will only propagate an array if the source is never
+               written to, and the only store to the target is the copy.
   --eliminate-common-uniform
                Perform load/load elimination for duplicate uniform values.
                Converts any constant index access chain uniform loads into
@@ -120,16 +144,16 @@ Options (in lexicographical order):
   --eliminate-dead-functions
                Deletes functions that cannot be reached from entry points or
                exported functions.
-  --eliminate-dead-insert
+  --eliminate-dead-inserts
                Deletes unreferenced inserts into composites, most notably
                unused stores to vector components, that are not removed by
                aggressive dead code elimination.
   --eliminate-dead-variables
                Deletes module scope variables that are not referenced.
   --eliminate-insert-extract
-               Replace extract from a sequence of inserts with the
-               corresponding value. Performed only on entry point call tree
-               functions.
+               DEPRECATED.  This pass has been replaced by the simplification
+               pass, and that pass will be run instead.
+               See --simplify-instructions.
   --eliminate-local-multi-store
                Replace stores and loads of function scope variables that are
                stored multiple times. Performed on variables referenceed only
@@ -162,33 +186,65 @@ Options (in lexicographical order):
                early return in a loop.
   --legalize-hlsl
                Runs a series of optimizations that attempts to take SPIR-V
-               generated by and HLSL front-end and generate legal Vulkan SPIR-V.
+               generated by an HLSL front-end and generates legal Vulkan SPIR-V.
                The optimizations are:
                %s
 
-               Note this does not guarantee legal code. This option implies
-               --skip-validation.
+               Note this does not guarantee legal code. This option passes the
+               option --relax-logical-pointer to the validator.
   --local-redundancy-elimination
                Looks for instructions in the same basic block that compute the
                same value, and deletes the redundant ones.
+  --loop-fission
+               Splits any top level loops in which the register pressure has
+               exceeded a given threshold. The threshold must follow the use of
+               this flag and must be a positive integer value.
+  --loop-fusion
+               Identifies adjacent loops with the same lower and upper bound.
+               If this is legal, then merge the loops into a single loop.
+               Includes heuristics to ensure it does not increase number of
+               registers too much, while reducing the number of loads from
+               memory. Takes an additional positive integer argument to set
+               the maximum number of registers.
+  --loop-invariant-code-motion
+               Identifies code in loops that has the same value for every
+               iteration of the loop, and move it to the loop pre-header.
   --loop-unroll
                Fully unrolls loops marked with the Unroll flag
   --loop-unroll-partial
                Partially unrolls loops marked with the Unroll flag. Takes an
                additional non-0 integer argument to set the unroll factor, or
                how many times a loop body should be duplicated
+  --loop-peeling
+               Execute few first (respectively last) iterations before
+               (respectively after) the loop if it can elide some branches.
+  --loop-peeling-threshold
+               Takes a non-0 integer argument to set the loop peeling code size
+               growth threshold. The threshold prevents the loop peeling
+               from happening if the code size increase created by
+               the optimization is above the threshold.
+  --max-id-bound=<n>
+               Sets the maximum value for the id bound for the moudle.  The
+               default is the minimum value for this limit, 0x3FFFFF.  See
+               section 2.17 of the Spir-V specification.
   --merge-blocks
                Join two blocks into a single block if the second has the
                first as its only predecessor. Performed only on entry point
                call tree functions.
   --merge-return
-               Replace all return instructions with unconditional branches to
-               a new basic block containing an unified return.
-               This pass does not currently support structured control flow. It
-               makes no changes if the shader capability is detected.
-  --local-redundancy-elimination
-               Looks for instructions in the same basic block that compute the
-               same value, and deletes the redundant ones.
+               Changes functions that have multiple return statements so they
+               have a single return statement.
+
+               For structured control flow it is assumed that the only
+               unreachable blocks in the function are trivial merge and continue
+               blocks.
+
+               A trivial merge block contains the label and an OpUnreachable
+               instructions, nothing else.  A trivial continue block contain a
+               label and an OpBranch to the header, nothing else.
+
+               These conditions are guaranteed to be met after running
+               dead-branch elimination.
   --loop-unswitch
                Hoists loop-invariant conditionals out of loops by duplicating
                the loop on each branch of the conditional and adjusting each
@@ -242,9 +298,9 @@ Options (in lexicographical order):
   --private-to-local
                Change the scope of private variables that are used in a single
                function to that function.
-  --remove-duplicates
-               Removes duplicate types, decorations, capabilities and extension
-               instructions.
+  --reduce-load-size
+               Replaces loads of composite objects where not every component is
+               used by loads of just the elements that are used.
   --redundancy-elimination
                Looks for instructions in the same function that compute the
                same value, and deletes the redundant ones.
@@ -252,14 +308,22 @@ Options (in lexicographical order):
                Allow store from one struct type to a different type with
                compatible layout and members. This option is forwarded to the
                validator.
+  --remove-duplicates
+               Removes duplicate types, decorations, capabilities and extension
+               instructions.
   --replace-invalid-opcode
                Replaces instructions whose opcode is valid for shader modules,
                but not for the current shader stage.  To have an effect, all
                entry points must have the same execution model.
-  --scalar-replacement
+  --ssa-rewrite
+               Replace loads and stores to function local variables with
+               operations on SSA IDs.
+  --scalar-replacement[=<n>]
                Replace aggregate function scope variables that are only accessed
                via their elements with new function variables representing each
-               element.
+               element.  <n> is a limit on the size of the aggragates that will
+               be replaced.  0 means there is no limit.  The default value is
+               100.
   --set-spec-const-default-value "<spec id>:<default value> ..."
                Set the default values of the specialization constants with
                <spec id>:<default value> pairs specified in a double-quoted
@@ -268,7 +332,7 @@ Options (in lexicographical order):
                be separated with colon ':' without any blank spaces in between.
                e.g.: --set-spec-const-default-value "1:100 2:400"
   --simplify-instructions
-               Will simplfy all instructions in the function as much as
+               Will simplify all instructions in the function as much as
                possible.
   --skip-validation
                Will not validate the SPIR-V before optimizing.  If the SPIR-V
@@ -278,6 +342,40 @@ Options (in lexicographical order):
                Replaces instructions with equivalent and less expensive ones.
   --strip-debug
                Remove all debug instructions.
+  --strip-reflect
+               Remove all reflection information.  For now, this covers
+               reflection information defined by SPV_GOOGLE_hlsl_functionality1.
+  --target-env=<env>
+               Set the target environment. Without this flag the target
+               enviroment defaults to spv1.3.
+               <env> must be one of vulkan1.0, vulkan1.1, opencl2.2, spv1.0,
+               spv1.1, spv1.2, spv1.3, or webgpu0.
+  --time-report
+               Print the resource utilization of each pass (e.g., CPU time,
+               RSS) to standard error output. Currently it supports only Unix
+               systems. This option is the same as -ftime-report in GCC. It
+               prints CPU/WALL/USR/SYS time (and RSS if possible), but note that
+               USR/SYS time are returned by getrusage() and can have a small
+               error.
+  --upgrade-memory-model
+               Upgrades the Logical GLSL450 memory model to Logical VulkanKHR.
+               Transforms memory, image, atomic and barrier operations to conform
+               to that model's requirements.
+  --vector-dce
+               This pass looks for components of vectors that are unused, and
+               removes them from the vector.  Note this would still leave around
+               lots of dead code that a pass of ADCE will be able to remove.
+  --webgpu-mode
+               Turns on the prescribed passes for WebGPU and sets the target
+               environmet to webgpu0. Other passes may be turned on via
+               additional flags, but such combinations are not tested.
+               Using --target-env with this flag is not allowed.
+
+               This flag is the equivalent of passing in --target-env=webgpu0
+               and specifying the following optimization code names:
+               %s
+
+               NOTE: This flag is a WIP and its behaviour is subject to change.
   --workaround-1209
                Rewrites instructions for which there are known driver bugs to
                avoid triggering those bugs.
@@ -290,7 +388,8 @@ Options (in lexicographical order):
                Display optimizer version information.
 )",
       program, program, GetLegalizationPasses().c_str(),
-      GetOptimizationPasses().c_str(), GetSizePasses().c_str());
+      GetOptimizationPasses().c_str(), GetSizePasses().c_str(),
+      GetWebGPUPasses().c_str());
 }
 
 // Reads command-line flags  the file specified in |oconfig_flag|. This string
@@ -304,7 +403,8 @@ bool ReadFlagsFromFile(const char* oconfig_flag,
                        std::vector<std::string>* file_flags) {
   const char* fname = strchr(oconfig_flag, '=');
   if (fname == nullptr || fname[0] != '=') {
-    fprintf(stderr, "error: Invalid -Oconfig flag %s\n", oconfig_flag);
+    spvtools::Errorf(opt_diagnostic, nullptr, {}, "Invalid -Oconfig flag %s",
+                     oconfig_flag);
     return false;
   }
   fname++;
@@ -312,14 +412,25 @@ bool ReadFlagsFromFile(const char* oconfig_flag,
   std::ifstream input_file;
   input_file.open(fname);
   if (input_file.fail()) {
-    fprintf(stderr, "error: Could not open file '%s'\n", fname);
+    spvtools::Errorf(opt_diagnostic, nullptr, {}, "Could not open file '%s'",
+                     fname);
     return false;
   }
 
-  while (!input_file.eof()) {
-    std::string flag;
-    input_file >> flag;
-    if (flag.length() > 0 && flag[0] != '#') {
+  std::string line;
+  while (std::getline(input_file, line)) {
+    // Ignore empty lines and lines starting with the comment marker '#'.
+    if (line.length() == 0 || line[0] == '#') {
+      continue;
+    }
+
+    // Tokenize the line.  Add all found tokens to the list of found flags. This
+    // mimics the way the shell will parse whitespace on the command line. NOTE:
+    // This does not support quoting and it is not intended to.
+    std::istringstream iss(line);
+    while (!iss.eof()) {
+      std::string flag;
+      iss >> flag;
       file_flags->push_back(flag);
     }
   }
@@ -327,26 +438,31 @@ bool ReadFlagsFromFile(const char* oconfig_flag,
   return true;
 }
 
-OptStatus ParseFlags(int argc, const char** argv, Optimizer* optimizer,
-                     const char** in_file, const char** out_file,
-                     spv_validator_options options, bool* skip_validator);
+OptStatus ParseFlags(int argc, const char** argv,
+                     spvtools::Optimizer* optimizer, const char** in_file,
+                     const char** out_file,
+                     spvtools::ValidatorOptions* validator_options,
+                     spvtools::OptimizerOptions* optimizer_options);
 
 // Parses and handles the -Oconfig flag. |prog_name| contains the name of
 // the spirv-opt binary (used to build a new argv vector for the recursive
 // invocation to ParseFlags). |opt_flag| contains the -Oconfig=FILENAME flag.
-// |optimizer|, |in_file| and |out_file| are as in ParseFlags.
+// |optimizer|, |in_file|, |out_file|, |validator_options|, and
+// |optimizer_options| are as in ParseFlags.
 //
 // This returns the same OptStatus instance returned by ParseFlags.
 OptStatus ParseOconfigFlag(const char* prog_name, const char* opt_flag,
-                           Optimizer* optimizer, const char** in_file,
-                           const char** out_file) {
+                           spvtools::Optimizer* optimizer, const char** in_file,
+                           const char** out_file,
+                           spvtools::ValidatorOptions* validator_options,
+                           spvtools::OptimizerOptions* optimizer_options) {
   std::vector<std::string> flags;
   flags.push_back(prog_name);
 
   std::vector<std::string> file_flags;
   if (!ReadFlagsFromFile(opt_flag, &file_flags)) {
-    fprintf(stderr,
-            "error: Could not read optimizer flags from configuration file\n");
+    spvtools::Error(opt_diagnostic, nullptr, {},
+                    "Could not read optimizer flags from configuration file");
     return {OPT_STOP, 1};
   }
   flags.insert(flags.end(), file_flags.begin(), file_flags.end());
@@ -354,32 +470,56 @@ OptStatus ParseOconfigFlag(const char* prog_name, const char* opt_flag,
   const char** new_argv = new const char*[flags.size()];
   for (size_t i = 0; i < flags.size(); i++) {
     if (flags[i].find("-Oconfig=") != std::string::npos) {
-      fprintf(stderr,
-              "error: Flag -Oconfig= may not be used inside the configuration "
-              "file\n");
+      spvtools::Error(
+          opt_diagnostic, nullptr, {},
+          "Flag -Oconfig= may not be used inside the configuration file");
       return {OPT_STOP, 1};
     }
     new_argv[i] = flags[i].c_str();
   }
 
-  bool skip_validator = false;
   return ParseFlags(static_cast<int>(flags.size()), new_argv, optimizer,
-                    in_file, out_file, nullptr, &skip_validator);
+                    in_file, out_file, validator_options, optimizer_options);
 }
 
-OptStatus ParseLoopUnrollPartialArg(int argc, const char** argv, int argi,
-                                    Optimizer* optimizer) {
-  if (argi < argc) {
-    int factor = atoi(argv[argi]);
-    if (factor != 0) {
-      optimizer->RegisterPass(CreateLoopUnrollPass(false, factor));
-      return {OPT_CONTINUE, 0};
+// Canonicalize the flag in |argv[argi]| of the form '--pass arg' into
+// '--pass=arg'. The optimizer only accepts arguments to pass names that use the
+// form '--pass_name=arg'.  Since spirv-opt also accepts the other form, this
+// function makes the necessary conversion.
+//
+// Pass flags that require additional arguments should be handled here.  Note
+// that additional arguments should be given as a single string.  If the flag
+// requires more than one argument, the pass creator in
+// Optimizer::GetPassFromFlag() should parse it accordingly (e.g., see the
+// handler for --set-spec-const-default-value).
+//
+// If the argument requests one of the passes that need an additional argument,
+// |argi| is modified to point past the current argument, and the string
+// "argv[argi]=argv[argi + 1]" is returned. Otherwise, |argi| is unmodified and
+// the string "|argv[argi]|" is returned.
+std::string CanonicalizeFlag(const char** argv, int argc, int* argi) {
+  const char* cur_arg = argv[*argi];
+  const char* next_arg = (*argi + 1 < argc) ? argv[*argi + 1] : nullptr;
+  std::ostringstream canonical_arg;
+  canonical_arg << cur_arg;
+
+  // NOTE: DO NOT ADD NEW FLAGS HERE.
+  //
+  // These flags are supported for backwards compatibility.  When adding new
+  // passes that need extra arguments in its command-line flag, please make them
+  // use the syntax "--pass_name[=pass_arg].
+  if (0 == strcmp(cur_arg, "--set-spec-const-default-value") ||
+      0 == strcmp(cur_arg, "--loop-fission") ||
+      0 == strcmp(cur_arg, "--loop-fusion") ||
+      0 == strcmp(cur_arg, "--loop-unroll-partial") ||
+      0 == strcmp(cur_arg, "--loop-peeling-threshold")) {
+    if (next_arg) {
+      canonical_arg << "=" << next_arg;
+      ++(*argi);
     }
   }
-  fprintf(stderr,
-          "error: --loop-unroll-partial must be followed by a non-0 "
-          "integer\n");
-  return {OPT_STOP, 1};
+
+  return canonical_arg.str();
 }
 
 // Parses command-line flags. |argc| contains the number of command-line flags.
@@ -390,14 +530,20 @@ OptStatus ParseLoopUnrollPartialArg(int argc, const char** argv, int argi,
 // The name of the output file in |out_file|. The return value indicates whether
 // optimization should continue and a status code indicating an error or
 // success.
-OptStatus ParseFlags(int argc, const char** argv, Optimizer* optimizer,
-                     const char** in_file, const char** out_file,
-                     spv_validator_options options, bool* skip_validator) {
+OptStatus ParseFlags(int argc, const char** argv,
+                     spvtools::Optimizer* optimizer, const char** in_file,
+                     const char** out_file,
+                     spvtools::ValidatorOptions* validator_options,
+                     spvtools::OptimizerOptions* optimizer_options) {
+  std::vector<std::string> pass_flags;
+  bool target_env_set = false;
+  bool webgpu_mode_set = false;
   for (int argi = 1; argi < argc; ++argi) {
     const char* cur_arg = argv[argi];
     if ('-' == cur_arg[0]) {
       if (0 == strcmp(cur_arg, "--version")) {
-        printf("%s\n", spvSoftwareVersionDetailsString());
+        spvtools::Logf(opt_diagnostic, SPV_MSG_INFO, nullptr, {}, "%s\n",
+                       spvSoftwareVersionDetailsString());
         return {OPT_STOP, 0};
       } else if (0 == strcmp(cur_arg, "--help") || 0 == strcmp(cur_arg, "-h")) {
         PrintUsage(argv[0]);
@@ -409,150 +555,100 @@ OptStatus ParseFlags(int argc, const char** argv, Optimizer* optimizer,
           PrintUsage(argv[0]);
           return {OPT_STOP, 1};
         }
-      } else if (0 == strcmp(cur_arg, "--strip-debug")) {
-        optimizer->RegisterPass(CreateStripDebugInfoPass());
-      } else if (0 == strcmp(cur_arg, "--set-spec-const-default-value")) {
-        if (++argi < argc) {
-          auto spec_ids_vals =
-              opt::SetSpecConstantDefaultValuePass::ParseDefaultValuesString(
-                  argv[argi]);
-          if (!spec_ids_vals) {
-            fprintf(stderr,
-                    "error: Invalid argument for "
-                    "--set-spec-const-default-value: %s\n",
-                    argv[argi]);
-            return {OPT_STOP, 1};
-          }
-          optimizer->RegisterPass(
-              CreateSetSpecConstantDefaultValuePass(std::move(*spec_ids_vals)));
-        } else {
-          fprintf(
-              stderr,
-              "error: Expected a string of <spec id>:<default value> pairs.");
-          return {OPT_STOP, 1};
-        }
-      } else if (0 == strcmp(cur_arg, "--if-conversion")) {
-        optimizer->RegisterPass(CreateIfConversionPass());
-      } else if (0 == strcmp(cur_arg, "--freeze-spec-const")) {
-        optimizer->RegisterPass(CreateFreezeSpecConstantValuePass());
-      } else if (0 == strcmp(cur_arg, "--inline-entry-points-exhaustive")) {
-        optimizer->RegisterPass(CreateInlineExhaustivePass());
-      } else if (0 == strcmp(cur_arg, "--inline-entry-points-opaque")) {
-        optimizer->RegisterPass(CreateInlineOpaquePass());
-      } else if (0 == strcmp(cur_arg, "--convert-local-access-chains")) {
-        optimizer->RegisterPass(CreateLocalAccessChainConvertPass());
-      } else if (0 == strcmp(cur_arg, "--eliminate-dead-code-aggressive")) {
-        optimizer->RegisterPass(CreateAggressiveDCEPass());
-      } else if (0 == strcmp(cur_arg, "--eliminate-insert-extract")) {
-        optimizer->RegisterPass(CreateInsertExtractElimPass());
-      } else if (0 == strcmp(cur_arg, "--eliminate-local-single-block")) {
-        optimizer->RegisterPass(CreateLocalSingleBlockLoadStoreElimPass());
-      } else if (0 == strcmp(cur_arg, "--eliminate-local-single-store")) {
-        optimizer->RegisterPass(CreateLocalSingleStoreElimPass());
-      } else if (0 == strcmp(cur_arg, "--merge-blocks")) {
-        optimizer->RegisterPass(CreateBlockMergePass());
-      } else if (0 == strcmp(cur_arg, "--merge-return")) {
-        optimizer->RegisterPass(CreateMergeReturnPass());
-      } else if (0 == strcmp(cur_arg, "--eliminate-dead-branches")) {
-        optimizer->RegisterPass(CreateDeadBranchElimPass());
-      } else if (0 == strcmp(cur_arg, "--eliminate-dead-functions")) {
-        optimizer->RegisterPass(CreateEliminateDeadFunctionsPass());
-      } else if (0 == strcmp(cur_arg, "--eliminate-local-multi-store")) {
-        optimizer->RegisterPass(CreateLocalMultiStoreElimPass());
-      } else if (0 == strcmp(cur_arg, "--eliminate-common-uniform")) {
-        optimizer->RegisterPass(CreateCommonUniformElimPass());
-      } else if (0 == strcmp(cur_arg, "--eliminate-dead-const")) {
-        optimizer->RegisterPass(CreateEliminateDeadConstantPass());
-      } else if (0 == strcmp(cur_arg, "--eliminate-dead-inserts")) {
-        optimizer->RegisterPass(CreateDeadInsertElimPass());
-      } else if (0 == strcmp(cur_arg, "--eliminate-dead-variables")) {
-        optimizer->RegisterPass(CreateDeadVariableEliminationPass());
-      } else if (0 == strcmp(cur_arg, "--fold-spec-const-op-composite")) {
-        optimizer->RegisterPass(CreateFoldSpecConstantOpAndCompositePass());
-      } else if (0 == strcmp(cur_arg, "--loop-unswitch")) {
-        optimizer->RegisterPass(CreateLoopUnswitchPass());
-      } else if (0 == strcmp(cur_arg, "--scalar-replacement")) {
-        optimizer->RegisterPass(CreateScalarReplacementPass());
-      } else if (0 == strcmp(cur_arg, "--strength-reduction")) {
-        optimizer->RegisterPass(CreateStrengthReductionPass());
-      } else if (0 == strcmp(cur_arg, "--unify-const")) {
-        optimizer->RegisterPass(CreateUnifyConstantPass());
-      } else if (0 == strcmp(cur_arg, "--flatten-decorations")) {
-        optimizer->RegisterPass(CreateFlattenDecorationPass());
-      } else if (0 == strcmp(cur_arg, "--compact-ids")) {
-        optimizer->RegisterPass(CreateCompactIdsPass());
-      } else if (0 == strcmp(cur_arg, "--cfg-cleanup")) {
-        optimizer->RegisterPass(CreateCFGCleanupPass());
-      } else if (0 == strcmp(cur_arg, "--local-redundancy-elimination")) {
-        optimizer->RegisterPass(CreateLocalRedundancyEliminationPass());
-      } else if (0 == strcmp(cur_arg, "--loop-invariant-code-motion")) {
-        optimizer->RegisterPass(CreateLoopInvariantCodeMotionPass());
-      } else if (0 == strcmp(cur_arg, "--redundancy-elimination")) {
-        optimizer->RegisterPass(CreateRedundancyEliminationPass());
-      } else if (0 == strcmp(cur_arg, "--private-to-local")) {
-        optimizer->RegisterPass(CreatePrivateToLocalPass());
-      } else if (0 == strcmp(cur_arg, "--remove-duplicates")) {
-        optimizer->RegisterPass(CreateRemoveDuplicatesPass());
-      } else if (0 == strcmp(cur_arg, "--workaround-1209")) {
-        optimizer->RegisterPass(CreateWorkaround1209Pass());
-      } else if (0 == strcmp(cur_arg, "--relax-struct-store")) {
-        options->relax_struct_store = true;
-      } else if (0 == strcmp(cur_arg, "--replace-invalid-opcode")) {
-        optimizer->RegisterPass(CreateReplaceInvalidOpcodePass());
-      } else if (0 == strcmp(cur_arg, "--simplify-instructions")) {
-        optimizer->RegisterPass(CreateSimplificationPass());
-      } else if (0 == strcmp(cur_arg, "--loop-unroll")) {
-        optimizer->RegisterPass(CreateLoopUnrollPass(true));
-      } else if (0 == strcmp(cur_arg, "--loop-unroll-partial")) {
-        OptStatus status =
-            ParseLoopUnrollPartialArg(argc, argv, ++argi, optimizer);
-        if (status.action != OPT_CONTINUE) {
-          return status;
-        }
-      } else if (0 == strcmp(cur_arg, "--skip-validation")) {
-        *skip_validator = true;
-      } else if (0 == strcmp(cur_arg, "-O")) {
-        optimizer->RegisterPerformancePasses();
-      } else if (0 == strcmp(cur_arg, "-Os")) {
-        optimizer->RegisterSizePasses();
-      } else if (0 == strcmp(cur_arg, "--legalize-hlsl")) {
-        *skip_validator = true;
-        optimizer->RegisterLegalizationPasses();
-      } else if (0 == strncmp(cur_arg, "-Oconfig=", sizeof("-Oconfig=") - 1)) {
-        OptStatus status =
-            ParseOconfigFlag(argv[0], cur_arg, optimizer, in_file, out_file);
-        if (status.action != OPT_CONTINUE) {
-          return status;
-        }
-      } else if (0 == strcmp(cur_arg, "--ccp")) {
-        optimizer->RegisterPass(CreateCCPPass());
-      } else if (0 == strcmp(cur_arg, "--print-all")) {
-        optimizer->SetPrintAll(&std::cerr);
       } else if ('\0' == cur_arg[1]) {
         // Setting a filename of "-" to indicate stdin.
         if (!*in_file) {
           *in_file = cur_arg;
         } else {
-          fprintf(stderr, "error: More than one input file specified\n");
+          spvtools::Error(opt_diagnostic, nullptr, {},
+                          "More than one input file specified");
           return {OPT_STOP, 1};
         }
+      } else if (0 == strncmp(cur_arg, "-Oconfig=", sizeof("-Oconfig=") - 1)) {
+        OptStatus status =
+            ParseOconfigFlag(argv[0], cur_arg, optimizer, in_file, out_file,
+                             validator_options, optimizer_options);
+        if (status.action != OPT_CONTINUE) {
+          return status;
+        }
+      } else if (0 == strcmp(cur_arg, "--skip-validation")) {
+        optimizer_options->set_run_validator(false);
+      } else if (0 == strcmp(cur_arg, "--print-all")) {
+        optimizer->SetPrintAll(&std::cerr);
+      } else if (0 == strcmp(cur_arg, "--time-report")) {
+        optimizer->SetTimeReport(&std::cerr);
+      } else if (0 == strcmp(cur_arg, "--relax-struct-store")) {
+        validator_options->SetRelaxStructStore(true);
+      } else if (0 == strncmp(cur_arg, "--max-id-bound=",
+                              sizeof("--max-id-bound=") - 1)) {
+        auto split_flag = spvtools::utils::SplitFlagArgs(cur_arg);
+        // Will not allow values in the range [2^31,2^32).
+        uint32_t max_id_bound =
+            static_cast<uint32_t>(atoi(split_flag.second.c_str()));
+
+        // That SPIR-V mandates the minimum value for max id bound but
+        // implementations may allow higher minimum bounds.
+        if (max_id_bound < kDefaultMaxIdBound) {
+          spvtools::Error(opt_diagnostic, nullptr, {},
+                          "The max id bound must be at least 0x3FFFFF");
+          return {OPT_STOP, 1};
+        }
+        optimizer_options->set_max_id_bound(max_id_bound);
+        validator_options->SetUniversalLimit(spv_validator_limit_max_id_bound,
+                                             max_id_bound);
+      } else if (0 == strncmp(cur_arg,
+                              "--target-env=", sizeof("--target-env=") - 1)) {
+        if (webgpu_mode_set) {
+          spvtools::Error(opt_diagnostic, nullptr, {},
+                          "Cannot use both --webgpu-mode and --target-env at "
+                          "the same time");
+          return {OPT_STOP, 1};
+        }
+        const auto split_flag = spvtools::utils::SplitFlagArgs(cur_arg);
+        const auto target_env_str = split_flag.second.c_str();
+        spv_target_env target_env;
+        if (!spvParseTargetEnv(target_env_str, &target_env)) {
+          spvtools::Error(opt_diagnostic, nullptr, {},
+                          "Invalid value passed to --target-env");
+          return {OPT_STOP, 1};
+        }
+        optimizer->SetTargetEnv(target_env);
+      } else if (0 == strcmp(cur_arg, "--webgpu-mode")) {
+        if (target_env_set) {
+          spvtools::Error(opt_diagnostic, nullptr, {},
+                          "Cannot use both --webgpu-mode and --target-env at "
+                          "the same time");
+          return {OPT_STOP, 1};
+        }
+
+        optimizer->SetTargetEnv(SPV_ENV_WEBGPU_0);
+        optimizer->RegisterWebGPUPasses();
       } else {
-        fprintf(
-            stderr,
-            "error: Unknown flag '%s'. Use --help for a list of valid flags\n",
-            cur_arg);
-        return {OPT_STOP, 1};
+        // Some passes used to accept the form '--pass arg', canonicalize them
+        // to '--pass=arg'.
+        pass_flags.push_back(CanonicalizeFlag(argv, argc, &argi));
+
+        // If we were requested to legalize SPIR-V generated from the HLSL
+        // front-end, skip validation.
+        if (0 == strcmp(cur_arg, "--legalize-hlsl")) {
+          validator_options->SetRelaxLogicalPointer(true);
+        }
       }
     } else {
       if (!*in_file) {
         *in_file = cur_arg;
       } else {
-        fprintf(stderr, "error: More than one input file specified\n");
+        spvtools::Error(opt_diagnostic, nullptr, {},
+                        "More than one input file specified");
         return {OPT_STOP, 1};
       }
     }
   }
 
+  if (!optimizer->RegisterPassesFromFlags(pass_flags)) {
+    return {OPT_STOP, 1};
+  }
+
   return {OPT_CONTINUE, 0};
 }
 
@@ -561,28 +657,25 @@ OptStatus ParseFlags(int argc, const char** argv, Optimizer* optimizer,
 int main(int argc, const char** argv) {
   const char* in_file = nullptr;
   const char* out_file = nullptr;
-  bool skip_validator = false;
 
   spv_target_env target_env = kDefaultEnvironment;
-  spv_validator_options options = spvValidatorOptionsCreate();
+
 
   spvtools::Optimizer optimizer(target_env);
-  optimizer.SetMessageConsumer([](spv_message_level_t level, const char* source,
-                                  const spv_position_t& position,
-                                  const char* message) {
-    std::cerr << StringifyMessage(level, source, position, message)
-              << std::endl;
-  });
+  optimizer.SetMessageConsumer(spvtools::utils::CLIMessageConsumer);
 
+  spvtools::ValidatorOptions validator_options;
+  spvtools::OptimizerOptions optimizer_options;
   OptStatus status = ParseFlags(argc, argv, &optimizer, &in_file, &out_file,
-                                options, &skip_validator);
+                                &validator_options, &optimizer_options);
+  optimizer_options.set_validator_options(validator_options);
 
   if (status.action == OPT_STOP) {
     return status.code;
   }
 
   if (out_file == nullptr) {
-    fprintf(stderr, "error: -o required\n");
+    spvtools::Error(opt_diagnostic, nullptr, {}, "-o required");
     return 1;
   }
 
@@ -591,28 +684,10 @@ int main(int argc, const char** argv) {
     return 1;
   }
 
-  if (!skip_validator) {
-    // Let's do validation first.
-    spv_context context = spvContextCreate(target_env);
-    spv_diagnostic diagnostic = nullptr;
-    spv_const_binary_t binary_struct = {binary.data(), binary.size()};
-    spv_result_t error =
-        spvValidateWithOptions(context, options, &binary_struct, &diagnostic);
-    if (error) {
-      spvDiagnosticPrint(diagnostic);
-      spvDiagnosticDestroy(diagnostic);
-      spvValidatorOptionsDestroy(options);
-      spvContextDestroy(context);
-      return error;
-    }
-    spvDiagnosticDestroy(diagnostic);
-    spvValidatorOptionsDestroy(options);
-    spvContextDestroy(context);
-  }
-
   // By using the same vector as input and output, we save time in the case
   // that there was no change.
-  bool ok = optimizer.Run(binary.data(), binary.size(), &binary);
+  bool ok =
+      optimizer.Run(binary.data(), binary.size(), &binary, optimizer_options);
 
   if (!WriteFile<uint32_t>(out_file, "wb", binary.data(), binary.size())) {
     return 1;
